package cn.kdan.pdf.tech.core.aspect;

import cn.kdan.pdf.tech.core.service.CompanyService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

@Aspect
@Component
public class ApiKeyAspect {
    @Autowired
    private CompanyService companyService;
    @Around("@annotation(preCheckApiKey)")
    public Object checkApiKey(ProceedingJoinPoint joinPoint, PreCheckApiKey preCheckApiKey) throws Throwable {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new IllegalStateException("Could not find request attributes");
        }
        HttpServletRequest request = attributes.getRequest();
        String open = request.getHeader("open");
        if(!StringUtils.isEmpty(open)){
            // 如果秘钥有效，继续执行方法
            return joinPoint.proceed();
        }
        // 从header中获取秘钥
        String secretKey = request.getHeader("secretKey");
        String publicKey = request.getHeader("publicKey");
        if (StringUtils.isEmpty(secretKey) || StringUtils.isEmpty(publicKey)) {
            throw new SecurityException("API Key is missing");
        }
        String companyId = companyService.isValidApiKey(publicKey,secretKey);
        // 检查秘钥是否有效，并检查有效期
        if (StringUtils.isEmpty(companyId)) {
            throw new SecurityException("Invalid or expired API Key");
        }
        // 如果秘钥有效，继续执行方法
        return joinPoint.proceed();
    }
}
