package cn.kdan.pdf.tech.core.aspect;

import cn.kdan.pdf.tech.core.params.EncryptParam;
import cn.kdan.pdf.tech.core.params.OemUserParam;
import cn.kdan.pdf.tech.core.utils.RSAUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

@Aspect
@Component
public class ParamDecryptAspect {
    @Around("@annotation(paramDecrypt)")
    public Object decryptParam(ProceedingJoinPoint joinPoint, ParamDecrypt paramDecrypt) throws Throwable {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (attributes == null) {
            throw new IllegalStateException("Could not find request attributes");
        }
        // 获取方法参数
        Object[] args = joinPoint.getArgs();
        // 假设所有参数都是需要解密的字符串
        for (int i = 0; i < args.length; i++) {
            if (args[i] instanceof String) {
                // 这里调用解密方法，这里只是一个示例
                try {
                    String decrypted = new RSAUtils().customDecrypt((String) args[i]);
                    args[i] = decrypted;
                } catch (Exception e) {
                    throw new SecurityException("Invalid param");
                }
            }
            if (args[i] instanceof EncryptParam) {
                // 这里调用解密方法，这里只是一个示例
                try {
                    EncryptParam node = ((EncryptParam) args[i]);
                    String decrypted = new RSAUtils().customDecrypt(node.getVerifyParam());
                    node.setVerifyParam(decrypted);
                    args[i] = node;
                } catch (Exception e) {
                    throw new SecurityException("Invalid param");
                }
            }
        }

        // 如果秘钥有效，继续执行方法
        return joinPoint.proceed(args);
    }
}
