package cn.kdan.pdf.tech.core.aspect;

import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.model.VppRTeamMemberRole;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import cn.kdan.pdf.tech.core.service.VppRTeamMemberRoleService;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.LocalVariableTableParameterNameDiscoverer;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

@Component
@Aspect
@Slf4j
public class PreCheckRoleAspect {

    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;

    @Around("@annotation(preCheckRole)")
    public Object checkRole(ProceedingJoinPoint joinPoint, PreCheckRole preCheckRole) throws Throwable {
        //获取允许调用接口的角色列表
        RoleEnum[] roleEnums = preCheckRole.roles();
        VppMember vppMember = vppMemberService.getCurrentUser();
        List<String> userRoles = Arrays.asList(vppMember.getRole().split(CommonConstant.STRING_SIGN_COMMA));
        boolean haveRole = false;
        String param;
        for (RoleEnum roleEnum : roleEnums) {
            //只要用户包含任意一种角色，则通过
            if (userRoles.contains(roleEnum.code())) {
                haveRole = Boolean.TRUE;
                break;
            }
        }
        if (!haveRole) {
            throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_CURRENT_USER_NO_PERMISSION);
        }
        if (!StringUtils.isBlank(preCheckRole.resourceId())) {
            //获取团队id
            param = this.parseExpression(joinPoint, preCheckRole.resourceId());
            List<VppRTeamMemberRole> list = vppRTeamMemberRoleService.listByMemberIdAndTeamId(vppMember.getId(), param);
            if (CollectionUtils.isEmpty(list)) {
                throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_CURRENT_USER_NO_PERMISSION);
            }
        }
        return joinPoint.proceed();
    }

    /**
     * 根据EL表达式获取参数值
     *
     * @param joinPoint  内置参数
     * @param expression 注解填写的EL表达式
     * @return 参数值
     */
    private String parseExpression(ProceedingJoinPoint joinPoint, String expression) {
        ExpressionParser parser = new SpelExpressionParser();
        EvaluationContext context = new StandardEvaluationContext();

        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        Object[] args = joinPoint.getArgs();

        LocalVariableTableParameterNameDiscoverer discoverer = new LocalVariableTableParameterNameDiscoverer();
        String[] paramNameArr = discoverer.getParameterNames(method);
        if (paramNameArr == null) {
            throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_CURRENT_USER_NO_PERMISSION);
        }

        for (int i = 0; i < paramNameArr.length; i++) {
            context.setVariable(paramNameArr[i], args[i]);
        }

        String param = parser.parseExpression(expression).getValue(context, String.class);
        log.info("API访问权限验证，验证参数：{}，参数值：{}", expression, param);

        return param;
    }

}
