package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.model.Directory;
import cn.kdan.pdf.tech.core.pojo.MSGroup;
import cn.kdan.pdf.tech.core.pojo.dto.AddSyncDTO;
import cn.kdan.pdf.tech.core.pojo.dto.DirectoryCreateDTO;
import cn.kdan.pdf.tech.core.pojo.dto.DirectoryUpdateDTO;
import cn.kdan.pdf.tech.core.pojo.dto.ScheduleSettingDTO;
import cn.kdan.pdf.tech.core.pojo.vo.DirectoryVO;
import cn.kdan.pdf.tech.core.service.DirectoryService;
import cn.kdan.pdf.tech.core.service.ScheduleSettingService;
import cn.kdan.pdf.tech.core.service.SyncService;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 目录模块
 */
@RestController
@RequestMapping("/directory")
public class DirectoryController {

    @Resource
    private DirectoryService directoryService;
    @Resource
    private SyncService syncService;
    @Resource
    private ScheduleSettingService scheduleSettingService;

    /**
     * 获取目录列表
     *
     * @return DirectoryVO
     */
    @GetMapping("/list")
    public ResultMap<List<DirectoryVO>> list() {
        List<DirectoryVO> directoryList = directoryService.list();
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, directoryList);
    }

    /**
     * 创建目录
     *
     * @param directoryVO 目录信息
     * @return Directory
     */
    @PostMapping("/create")
    public ResultMap<Directory> create(@RequestBody @Valid DirectoryCreateDTO directoryVO) {
        Directory createdDirectory = directoryService.create(directoryVO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, createdDirectory);
    }

    /**
     * 删除目录
     *
     * @param directoryId 目录id
     * @return String
     */
    @PostMapping("/delete")
    public ResultMap<String> delete(@RequestParam String directoryId) {
        directoryService.delete(directoryId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 更新目录
     *
     * @param updateDTO 目录信息
     * @return Directory
     */
    @PostMapping("/update")
    public ResultMap<String> create(@RequestBody @Valid DirectoryUpdateDTO updateDTO) {
        directoryService.update(updateDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 添加同步设置
     *
     * @param addSyncDTO 同步设置
     * @return String
     */
    @PostMapping("/addSyncSettings")
    public ResultMap<String> addSyncSettings(@RequestBody @Valid AddSyncDTO addSyncDTO) {
        syncService.addSyncSettings(addSyncDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 获取微软组列表
     *
     * @param clientId     clientId
     * @param tenantId     tenantId
     * @param clientSecret clientSecret
     * @return List
     */
    @GetMapping("/getGroupList")
    public ResultMap<List<MSGroup>> getGroupList(@RequestParam String clientId, @RequestParam String tenantId, @RequestParam String clientSecret,  @RequestParam(required = false) String queryString) {
        List<MSGroup> directoryList = syncService.getGroupList(clientId, tenantId, clientSecret,queryString);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, directoryList);
    }

    /**
     * 同步(点击同步按钮之后)
     *
     * @param scheduleSettingDTO 同步设置
     * @return String
     */
    @PostMapping("/syncSetting")
    public ResultMap<String> syncSetting(@RequestBody @Valid ScheduleSettingDTO scheduleSettingDTO) {
        scheduleSettingService.syncSetting(scheduleSettingDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }
}
