package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.model.Domain;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.service.DomainService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;
import utils.CommonUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

/**
 * 域名模块
 */
@RestController
@RequestMapping("/domain")
public class DomainController {

    @Resource
    private DomainService domainService;
    @Resource
    private VppMemberService memberService;

    /**
     * 获取DNS的txt文件
     *
     * @return string
     */
    @GetMapping("/getDNSTxt")
    public ResultMap<String> getDNSTxt() {
        VppMember member = memberService.getCurrentUser();
        String txt = "PDF-Tech-Editor" + "-domain-verification=" + member.getCompanyId();
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, txt);
    }

    /**
     * 获取https验证文件
     *
     * @param response file
     */
    @GetMapping("/getHttpTxt")
    public void getHttpTxt(HttpServletResponse response) {
        String fileName = "PDF-Tech-Editor-domain-verification.txt";
        VppMember member = memberService.getCurrentUser();
        String content = "PDF-Tech-Editor-domain-verification=" + member.getCompanyId();

        response.setContentType("text/plain");
        response.setHeader("Content-Disposition", "attachment; filename=" + fileName);

        try (PrintWriter writer = response.getWriter()) {
            writer.print(content);
            writer.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取域名列表
     *
     * @param directoryId 目录id
     * @return list
     */
    @GetMapping("/listByDirectoryId")
    public ResultMap<List<Domain>> listByDirectoryId(@RequestParam String directoryId) {
        List<Domain> domainList = domainService.listByDirectoryId(directoryId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, domainList);
    }

    /**
     * 获取域名列表
     *
     * @param verified 0 未验证 1 已验证
     * @return list
     */
    @GetMapping("/list")
    public ResultMap<List<Domain>> list(@RequestParam(required = false) String verified,@RequestParam(required = false) String used) {
        List<Domain> domainList = domainService.list(verified,used);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, domainList);
    }

    /**
     * 删除域名
     *
     * @param domainId 域名id
     * @return string
     */
    @PostMapping("/delete")
    public ResultMap<String> delete(@RequestParam String domainId) {
        domainService.delete(domainId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 创建域名
     *
     * @param verifyType 验证类型
     * @param name       域名
     * @return string
     */
    @PostMapping("/create")
    public ResultMap<String> create(@RequestParam String verifyType, @RequestParam String name) {
        domainService.create(verifyType, name);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 验证域名
     *
     * @param domainId    域名id
     * @return string
     */
    @PostMapping("/verify")
    public ResultMap<String> verify(@RequestParam String domainId,@RequestParam String type) {
        domainService.verify(domainId,type);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 绑定域名
     *
     * @param domainId    域名id
     * @param directoryId 目录id
     * @return string
     */
    @PostMapping("/bind")
    public ResultMap<String> bind(@RequestParam String domainId, @RequestParam String directoryId) {
        domainService.bindDomain(domainId, directoryId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    @PostMapping("/unBind")
    public ResultMap<String> unBind(@RequestParam String domainId) {
        domainService.unBind(domainId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

}
