package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.ParamDecrypt;
import cn.kdan.pdf.tech.core.aspect.PreCheckApiKey;
import cn.kdan.pdf.tech.core.constant.OEMConstant;
import cn.kdan.pdf.tech.core.params.EncryptParam;
import cn.kdan.pdf.tech.core.params.OemUserParam;
import cn.kdan.pdf.tech.core.params.VerifyParam;
import cn.kdan.pdf.tech.core.pojo.oemVo.OemMemberVO;
import cn.kdan.pdf.tech.core.pojo.vo.ActivationVO;
import cn.kdan.pdf.tech.core.service.CompanyService;
import cn.kdan.pdf.tech.core.service.OEMService;
import cn.kdan.pdf.tech.core.utils.RSAUtils;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.JsonNode;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;
import utils.JsonUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import java.util.List;


/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/oem")
public class OEMController {

    @Resource
    private OEMService oemService;
    @Resource
    private CompanyService companyService;

    /**
     * 注册或者购买会员后更新序列码时长
     *
     * @return STRING
     */
    @PreCheckApiKey
    @ParamDecrypt
    @PostMapping("/operateOEMUser")
    public ResultMap<OemMemberVO> operateOEMUser(HttpServletRequest request, @RequestBody EncryptParam encryptParam) {
        OemUserParam param = JsonUtils.jsonStringToBean(encryptParam.getVerifyParam(), OemUserParam.class);
        String publicKey = request.getHeader("PublicKey");
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, oemService.operateOEMUser(param.getEmail(), companyService.getCurrentCompany(publicKey), param.getProductCode(), param.getPrice()));
    }

    /**
     * 删除序列码 (用于退单或者用户注销)
     * @param email 邮箱
     * @return STRING
     */
    @PreCheckApiKey
    @PostMapping("/deleteOEMUser")
    public ResultMap<String> deleteOEMUser(HttpServletRequest request, @NotBlank(message = "email不能为空") @RequestParam String email) {
        String publicKey = request.getHeader("PublicKey");
        oemService.deleteOEMUser(email, companyService.getCurrentCompany(publicKey));
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    /**
     * 激活序列码
     * @param encryptParam 验证参数
     * @return ActivationVO
     */
    @ParamDecrypt
    @PostMapping("/activate")
    public ResultMap<ActivationVO> activate(@RequestBody EncryptParam encryptParam) {
        VerifyParam param = JsonUtils.jsonStringToBean(encryptParam.getVerifyParam(), VerifyParam.class);
        return new ResultMap<>(CommonConstant.SUCCESS, OEMConstant.SUCCESS_ACTIVATION_SUCCESS,oemService.activation(param));
    }

    /**
     * 验证设备信息（打开设备时调用）
     * @param encryptParam 验证参数
     * @return ActivationVO
     */
    @ParamDecrypt
    @PostMapping("/verify")
    public ResultMap<ActivationVO> verify(@RequestBody EncryptParam encryptParam) {
        VerifyParam param = JsonUtils.jsonStringToBean(encryptParam.getVerifyParam(), VerifyParam.class);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,oemService.verify(param));
    }

    /**
     * 登出时使用取消序列码
     * @param encryptParam 需要设备uuid和appcode
     * @return String
     */
    @ParamDecrypt
    @PostMapping("/cancel")
    public ResultMap<String> cancel(@RequestBody EncryptParam encryptParam) {
        VerifyParam param = JsonUtils.jsonStringToBean(encryptParam.getVerifyParam(), VerifyParam.class);
        oemService.cancel(param);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

    @PostMapping("/encrypt")
    public ResultMap<String> encrypt(@RequestBody Object content) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, new RSAUtils().customEncrypt(JSON.toJSONString(content)));
    }

    @PostMapping("/decrypt")
    public ResultMap<String> decrypt(@RequestBody JsonNode node) {
        String content = node.get("content").asText();
        System.out.println(content);
    return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, new RSAUtils().customDecrypt(content));
    }

    @PostMapping("/generateCode")
    public ResultMap<List<String>> generateCode(String companyId, String productCode, Integer number) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, oemService.generateCode(companyId,productCode,number));
    }

}
