package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.pojo.vo.*;
import cn.kdan.pdf.tech.core.service.VppDashboardService;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author Bob
 */
@RestController
@RequestMapping("/vppDashboard")
public class VppDashboardController {

    @Resource
    private VppDashboardService vppDashboardService;

    /**
     * 获取全局预览数据，不区分产品
     * 不区分账户权限
     *
     * @return DashboardOverviewVO
     */
    @GetMapping("/getOverview")
    public ResultMap<DashboardOverviewVO> getOverview() {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppDashboardService.getOverview());
    }

    /**
     * 获取全局预览数据，区分产品
     * 区分超管和团队管理员权限
     *
     * @param codeList 产品编码
     * @return DashboardOverviewVO
     */
    @GetMapping("/getOverviewProduct")
    public ResultMap<DashboardOverviewProductVO> getOverviewProduct(@RequestParam(required = false) List<String> codeList) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppDashboardService.getOverviewProduct(codeList));
    }

    /**
     * 获取图表数据
     * 不区分账户权限
     *
     * @param type 1today,2last 7 days,3last 30 days,4this month
     * @return DashboardChartStatisticVO
     */
    @GetMapping("/getChartStatistics/{type}")
    public ResultMap<List<DashboardChartStatisticVO>> getChartStatistics(@PathVariable("type") Integer type) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppDashboardService.getChartStatistics(type));
    }

    /**
     * 获取团队数据
     * 不区分账户权限
     *
     * @return DashboardTeamDataVO
     */
    @GetMapping("/getTeamData")
    public ResultMap<DashboardTeamDataVO> getTeamData() {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppDashboardService.getTeamData());
    }

    /**
     * 获取团队成员数据
     * 区分超管和团队管理员权限
     *
     * @return DashboardTeamMemberVO
     */
    @GetMapping("/getTeamMemberData")
    public ResultMap<DashboardTeamMemberVO> getTeamMemberData(@RequestParam(required = false) List<String> codeList, @RequestParam(required = false) String teamId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppDashboardService.getTeamMemberData(codeList, teamId));
    }

}
