package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.PreCheckRole;
import cn.kdan.pdf.tech.core.constant.VppLicenseCodeConstant;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.model.LicenseCodes;
import cn.kdan.pdf.tech.core.model.VppCompany;
import cn.kdan.pdf.tech.core.pojo.vo.VppLicenseCodeVO;
import cn.kdan.pdf.tech.core.service.VppCompanyService;
import cn.kdan.pdf.tech.core.service.VppLicenseCodeService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pojo.ResultMap;
import utils.CommonUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppLicenseCode")
public class VppLicenseCodeController {

    @Resource
    private VppLicenseCodeService vppLicenseCodeService;
    @Resource
    private VppCompanyService vppCompanyService;

    /**
     * 序列码相关信息分页查询
     *
     * @param page        页数
     * @param pageSize    每页条数
     * @param productId   产品id
     * @param teamId      团队id
     * @param status      状态
     * @param queryString 邮箱用户名或者
     * @return PageInfo
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @GetMapping("/page")
    public ResultMap<PageInfo<VppLicenseCodeVO>> page(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                      @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                      @RequestParam(required = false) Integer productId,
                                                      @RequestParam(required = false) String teamId,
                                                      @RequestParam(required = false) Integer status,
                                                      @RequestParam(required = false) String queryString
    ) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppLicenseCodeService.page(page, pageSize, productId, teamId, status, queryString));
    }

    /**
     * 分销商序列码相关信息分页查询
     *
     * @param page        页数
     * @param pageSize    每页条数
     * @param productId   产品id
     * @param status      状态
     * @param queryString 序列码
     * @return PageInfo
     */
    @GetMapping("/pageForReseller")
    public ResultMap<PageInfo<VppLicenseCodeVO>> pageForReseller(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                                 @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                                 @RequestParam(required = false) Integer productId,
                                                                 @RequestParam(required = false) Integer status,
                                                                 @RequestParam(required = false) String queryString
    ) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppLicenseCodeService.pageForReseller(page, pageSize, productId, status, queryString));
    }


    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @GetMapping("/download")
    public void downloadFailedUsingJson(HttpServletResponse response,
                                        @RequestParam(required = false) Integer productId,
                                        @RequestParam(required = false) String teamId,
                                        @RequestParam(required = false) Integer status,
                                        @RequestParam(required = false) String queryString) throws IOException {
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        try {
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            VppCompany company = vppCompanyService.getUserCompany();
            String fileName = URLEncoder.encode(company.getName() + " " + CommonUtils.getDateTime("yyyyMMdd") + " License", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            // 这里需要设置不关闭流
            EasyExcel.write(response.getOutputStream(), VppLicenseCodeVO.class)
                    .autoCloseStream(Boolean.FALSE)
                    .sheet("模板")
                    .doWrite(() -> {
                        // 分页查询数据
                        return vppLicenseCodeService.listWithInfo(productId, teamId, status, queryString);
                    });
        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            response.getWriter().println(JSON.toJSONString(map));
        }
    }


    /**
     * 给指定用户授权指定产品的序列码
     *
     * @param memberIds  用户id集合
     * @param teamId     团队id
     * @param productIds 产品id集合
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @PostMapping("/singleAssign")
    public ResultMap<Boolean> singleAssign(@RequestParam List<String> memberIds,
                                           @RequestParam String teamId,
                                           @RequestParam List<Integer> productIds) {
        vppLicenseCodeService.singleAssign(memberIds, teamId, productIds);
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.SUCCESS_MSG_ASSIGN, Boolean.TRUE);
    }


    /**
     * 给团队中除了超级管理员外的所有用户授权指定产品的序列码
     *
     * @param teamId     团队id
     * @param productIds 产品id集合
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @PostMapping("/teamAssign")
    public ResultMap<Boolean> teamAssign(@RequestParam String teamId,
                                         @RequestParam List<Integer> productIds) {
        vppLicenseCodeService.teamAssign(teamId, productIds);
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.SUCCESS_MSG_ASSIGN, Boolean.TRUE);
    }

    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @PostMapping("/cancelLicenceAssign")
    public ResultMap<Boolean> cancelLicenceAssign(@RequestParam String licenceId) {
        vppLicenseCodeService.cancelLicenceAssign(licenceId);
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.CANCEL_ASSIGN, Boolean.TRUE);
    }

    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @PostMapping("/cancelLicenceAssignBatch")
    public ResultMap<String> cancelLicenceAssignBatch(@RequestParam("file") MultipartFile file) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.CANCEL_ASSIGN, vppLicenseCodeService.cancelLicenceAssignBatch(file));
    }

    /**
     * 检查公司有没有剩余序列码
     *
     * @param companyId
     * @return
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @GetMapping("/checkCompanyLicense")
    public ResultMap<Boolean> checkCompanyLicense(@RequestParam("companyId") String companyId) {
        List<LicenseCodes> list = vppLicenseCodeService.getByCompanyIdRemoveExpired(companyId);
        boolean flag = !CollectionUtils.isEmpty(list);
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.SUCCESS_MSG_ASSIGN, flag);
    }

    /**
     * 检查公司有没有剩余序列码
     *
     * @param licenseCodeId 序列码id
     * @return 禁用
     */
    @PostMapping("/disable")
    public ResultMap<String> disable(@RequestParam("licenseCodeId") String licenseCodeId) {
        vppLicenseCodeService.disable(licenseCodeId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }

}
