package cn.kdan.pdf.tech.core.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.SSOConstant;
import cn.kdan.pdf.tech.core.constant.VppLicenseCodeConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.model.Directory;
import cn.kdan.pdf.tech.core.model.LicenseCodes;
import cn.kdan.pdf.tech.core.model.VppCompany;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.params.VppMemberRegisterParam;
import cn.kdan.pdf.tech.core.params.VppMemberResetPwdParam;
import cn.kdan.pdf.tech.core.params.VppMemberUpdatePwdParam;
import cn.kdan.pdf.tech.core.pojo.vo.SSOClientVO;
import cn.kdan.pdf.tech.core.pojo.vo.SSOMemberLicenseVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppMemberVO;
import cn.kdan.pdf.tech.core.service.*;
import constant.CommonConstant;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Date;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppMember")
public class VppMemberController {

    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppLicenseCodeService licenseCodeService;
    @Resource
    private DirectoryService directoryService;
    @Resource
    private VppDeviceService deviceService;
    @Resource
    private VppCompanyService vppCompanyService;

    /**
     * 用户注册
     *
     * @param param 用户注册入参
     * @return 用户注册出参
     */
    @PostMapping("/create")
    public ResultMap<VppMember> create(@RequestBody VppMemberRegisterParam param) {
        return new ResultMap<>(CommonConstant.SUCCESS, AuthConstant.SUCCESS_MSG_Register, vppMemberService.create(param));
    }

    /**
     * 试用超管注册
     *
     * @param param 用户注册入参
     * @return 用户注册出参
     */
    @PostMapping("/createTrial")
    public ResultMap<Boolean> createTrial(@RequestBody VppMemberRegisterParam param) {
        return new ResultMap<>(CommonConstant.SUCCESS, AuthConstant.SUCCESS_MSG_Register, vppMemberService.createTrial(param));
    }

    /**
     * 用于忘记密码时重置密码
     *
     * @param params params
     * @return Boolean
     */
    @PostMapping("resetPassword")
    public ResultMap<Boolean> resetPassword(@RequestBody VppMemberResetPwdParam params) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppMemberConstant.PASSWORD_RESET_SUCCESS, vppMemberService.resetPassword(params));
    }


    /**
     * 用户修改密码
     *
     * @param params params
     * @return Boolean
     */
    @PostMapping("updatePassword")
    public ResultMap<Boolean> updatePassword(@RequestBody @Valid VppMemberUpdatePwdParam params) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppMemberConstant.PASSWORD_UPDATE_SUCCESS, vppMemberService.updatePassword(params));
    }

    /**
     * 添加管理员时判断用户是否存在
     *
     * @param account 邮箱
     * @return Boolean
     */
    @PostMapping("/checkExist")
    public ResultMap<Boolean> checkExist(@RequestParam String account) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppMemberService.checkExist(account));
    }

    /**
     * 获取当前用户信息
     *
     * @return VppMember
     */
    @GetMapping("/getMemberInfo")
    public ResultMap<VppMemberVO> getMemberInfo() {
        VppMember vppMember = vppMemberService.getCurrentUser();
        VppMemberVO vppMemberVO = new VppMemberVO();
        VppMember byId = vppMemberService.getById(vppMember.getId());
        BeanUtil.copyProperties(byId, vppMemberVO);
        vppMemberVO.setDigestPassword(null);
        VppCompany company = vppCompanyService.getById(byId.getCompanyId());
        vppMemberVO.setCompany(company.getName());
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppMemberVO);
    }

    /**
     * 检查单点登录用户（app端使用）
     * @param email 邮箱
     * @param productCode 产品code
     * @return SSOMemberLicenseVO
     */
    @GetMapping("/checkSSOMember")
    public ResultMap<SSOMemberLicenseVO> checkSSOMember(@RequestParam("email") String email, @RequestParam("productCode") String productCode) {
        VppMember member = vppMemberService.getByAccountSSO(email);
        if (ObjectUtils.isEmpty(member)) {
            throw new BackendRuntimeException(SSOConstant.EXCEPTION_MSG_MEMBER_NOT_EXIST);
        }
        if (member.getValidFlag().equals(ValidStatusEnum.REVOKED.value())) {
            throw new BackendRuntimeException(SSOConstant.EXCEPTION_MSG_MEMBER_REVOKED);
        }
        LicenseCodes codes = licenseCodeService.getByMemberId(member.getId(), productCode);
        SSOMemberLicenseVO ssoMemberLicenseVO = new SSOMemberLicenseVO();
        BeanUtils.copyProperties(member, ssoMemberLicenseVO);
        if (!ObjectUtils.isEmpty(codes)) {
            BeanUtils.copyProperties(codes, ssoMemberLicenseVO);
            ssoMemberLicenseVO.setType(codes.getType().toString());
        }
        member.setLoginTime(new Date());
        member.setAdminConsoleLoginTime(new Date());
        vppMemberService.update(member);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, ssoMemberLicenseVO);
    }

    /**
     * 获取微软配置
     * @param email 邮箱
     * @return SSOClientVO
     */
    @GetMapping("/getSSOInfo")
    public ResultMap<SSOClientVO> getSSOInfo(@RequestParam("email") String email) {
        VppMember member = vppMemberService.getByAccountSSO(email);
        if (ObjectUtils.isEmpty(member)) {
            throw new BackendRuntimeException(SSOConstant.EXCEPTION_MSG_MEMBER_NOT_EXIST);
        }
        if (member.getValidFlag().equals(ValidStatusEnum.REVOKED.value())||member.getValidFlag().equals(ValidStatusEnum.INVALID.value())) {
            throw new BackendRuntimeException(SSOConstant.EXCEPTION_MSG_MEMBER_REVOKED);
        }
        Directory directory = directoryService.getById(member.getDirectoryId());
        SSOClientVO vo = new SSOClientVO();
        BeanUtils.copyProperties(directory, vo);
        return new ResultMap<>(CommonConstant.SUCCESS, VppLicenseCodeConstant.SUCCESS_MSG_ASSIGN, vo);
    }

    /**
     * SSO用户登出
     * @param deviceId 设备id
     * @return Boolean
     */
    @PostMapping("cancelSSOMember")
    public ResultMap<Boolean> cancelSSOMember(@RequestParam String deviceId) {
        deviceService.cancelSSO(deviceId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, true);
    }

    /**
     * 验证是否首次登录 true 是首次 false不是首次
     * @return Boolean
     */

    @GetMapping("/verifyFirstTimeLogin")
    public ResultMap<Boolean> verifyFirstTimeLogin() {
        VppMember currentUser = vppMemberService.getCurrentUser();
        //第一次登录之后才有时间
        VppMember member = new VppMember();
        member.setId(currentUser.getId());
        member.setAdminConsoleLoginTime(new Date());
        vppMemberService.update(member);
        if (ObjectUtils.isEmpty(currentUser.getAdminConsoleLoginTime())){
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
        }
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.FALSE);
    }


}
