package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.PreCheckRole;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.pojo.vo.SubscriptionResellerVO;
import cn.kdan.pdf.tech.core.pojo.vo.SubscriptionVO;
import cn.kdan.pdf.tech.core.service.SubscriptionService;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import pojo.ResultMap;

import javax.annotation.Resource;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppOrderDetail")
public class VppOrderDetailController {

    @Resource
    private SubscriptionService subscriptionService;

    /**
     * 获取当前登录用户所在公司的产品购买情况
     *
     * @param page     页码
     * @param pageSize 最大条数
     * @param code     code
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN, RoleEnum.SUPER_ADMIN})
    @GetMapping("/pageByCompany")
    public ResultMap<PageInfo<SubscriptionVO>> pageByCompany(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                             @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                             @RequestParam(required = false) String code) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, subscriptionService.pageByCompany(page, pageSize, code));
    }

    /**
     * 获取当前登录分销商所在公司的产品购买情况
     *
     * @param page     页码
     * @param pageSize 最大条数
     * @param code     code
     * @return ResultMap
     */
    @GetMapping("/pageForReseller")
    public ResultMap<PageInfo<SubscriptionResellerVO>> pageForReseller(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                                       @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                                       @RequestParam(required = false) String code) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, subscriptionService.pageForReseller(page, pageSize, code));
    }

}
