package cn.kdan.pdf.tech.core.service;

import cn.kdan.pdf.tech.core.model.Domain;

import java.util.List;

/**
 * 域名服务类
 */
public interface DomainService {
    /**
     * 根据目录id获取域名列表
     *
     * @param directoryId 目录id
     * @return 域名列表
     */
    List<Domain> listByDirectoryId(String directoryId);

    /**
     * 根据公司id获取域名列表(已验证的)
     *
     * @param verified 是否已验证
     * @param used
     * @return 域名列表
     */
    List<Domain> list(String verified, String used);

    /**
     * 删除域名
     *
     * @param domainId 域名id
     * @return bool
     */
    void delete(String domainId);

    void unbindByDirectoryId(String directoryId);

    /**
     * 创建域名
     *
     * @param verifyType 验证类型
     * @param name       域名
     */
    void create(String verifyType,String name);

    /**
     * 验证域名
     *
     * @param domainId    域名id
     */
    void verify(String domainId,String type);
    /**
     * 绑定域名
     *
     * @param domainId    域名id
     * @param directoryId 目录id
     */
    void bindDomain(String domainId, String directoryId);

    /**
     * 解绑域名
     * @param domainId 域名id
     */
    void unBind(String domainId);
}
