package cn.kdan.pdf.tech.core.service;

import cn.kdan.pdf.tech.core.model.Devices;
import cn.kdan.pdf.tech.core.pojo.vo.VppDeviceVO;
import com.github.pagehelper.PageInfo;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author tangxiangan
 */
public interface VppDeviceService {


    /**
     * 分页查询团队设备信息
     *
     * @param page      页码
     * @param pageSize  最大条数
     * @param productId 产品
     * @param status    设备状态
     * @param teamId    团队id
     * @return PageInfo
     */
    PageInfo<VppDeviceVO> page(Integer page, Integer pageSize, Integer productId, Integer status, String teamId, String queryString);

    /**
     * 团队设备信息列表（用于导出）
     *
     * @param productId 产品id
     * @param status 设备状态
     * @param teamId 团队id
     * @param queryString 账号或者邮箱
     * @return List
     */
    List<VppDeviceVO> listWithInfo(Integer productId, Integer status, String teamId, String queryString);

    /**
     * 解绑用户绑定的所有设备
     *
     * @param memberId 用户id
     * @param cdKeys   序列码集合
     */
    void unBindByMemberId(String memberId, List<String> cdKeys);

    /**
     * 解绑序列码和设备
     *
     * @param deviceId 设备id
     */
    void cancel(String deviceId);

    /**
     * 获取序列激活的设备
     * @param cdKeyList 序列码
     * @return
     */
    List<Devices> getByCdKeyList(List<String> cdKeyList);

    /**
     * 解绑SSO设备
     * @param uniqueSn 设备唯一码
     */
    void cancelSSO(String uniqueSn);
}
