package cn.kdan.pdf.tech.core.service;

import cn.kdan.pdf.tech.core.model.LicenseCodes;
import cn.kdan.pdf.tech.core.pojo.vo.VppLicenseCodeVO;
import com.github.pagehelper.PageInfo;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author tangxiangan
 */
public interface VppLicenseCodeService {


    /**
     * 解绑用户下的所有序列码
     *
     * @param teamIds  团队id集合（不传则解绑所有团队的序列码）
     * @param memberId 用户id
     */
    void unBindByMemberId(String memberId, List<String> teamIds);


    /**
     * 序列码相关信息分页查询
     *
     * @param page        页数
     * @param pageSize    每页条数
     * @param productId   产品id
     * @param teamId      团队id
     * @param status      状态
     * @param queryString 邮箱用户名或者
     * @return PageInfo
     */
    PageInfo<VppLicenseCodeVO> page(Integer page, Integer pageSize, Integer productId, String teamId,
                                    Integer status, String queryString);

    /**
     * 序列码列表
     *
     * @param productId
     * @param teamId
     * @param status
     * @param queryString
     * @return
     */
    List<VppLicenseCodeVO> listWithInfo(Integer productId, String teamId, Integer status, String queryString);

    /**
     * 给指定用户授权指定产品的序列码
     *
     * @param memberIds  用户id集合
     * @param teamId     团队id
     * @param productIds 产品id集合
     */
    void singleAssign(List<String> memberIds, String teamId, List<Integer> productIds);

    /**
     * 给团队中除了超级管理员外的所有用户授权指定产品的序列码
     *
     * @param teamId     团队id
     * @param productIds 产品id集合
     */
    void teamAssign(String teamId, List<Integer> productIds);

    void cancelLicenceAssign(String licenceId);

    String cancelLicenceAssignBatch(MultipartFile file);

    /**
     * 更新
     *
     * @param licenseCode licenseCode
     */
    void update(LicenseCodes licenseCode);

    void updateByPrimaryKey(LicenseCodes licenseCode);

    /**
     * 获取公司下的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByCompanyId(String companyId);

    /**
     * 获取公司下的未过期序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByCompanyIdRemoveExpired(String companyId);

    /**
     * 获取团队下的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByCompanyAndTeamId(String companyId, String teamId);

    /**
     * 获取团队下的所有已授权序列码
     * @param teamId 团队id
     * @return
     */
    List<LicenseCodes> getByTeamId(String teamId);

    /**
     * 获取团队下对应产品的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByTeamIdAndProduceId(String companyId, String teamId, Integer productId);

    /**
     * 获取多个团队下的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByTeamIds(String companyId, List<String> teamIdList);

    /**
     * 获取多个团队下对应多个产品的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByTeamIdsAndProduceIds(String companyId, List<String> teamIdList, List<Integer> productIdList);

    /**
     * 获取公司对应产品的所有序列码
     * @param companyId
     * @return
     */
    List<LicenseCodes> getByProduceIds(String companyId, List<Integer> productIdList);


    /**
     * 获取对应订阅产品的所有序列码
     * @param subscriptionIds 订阅ids
     * @return List
     */
    List<LicenseCodes> getBySubscriptionIds(List<String> subscriptionIds);


    LicenseCodes getByCdKey(String cdKey);

    /**
     * 分销商序列码相关信息分页查询
     *
     * @param page        页数
     * @param pageSize    每页条数
     * @param productId   产品id
     * @param status      状态
     * @param queryString 邮箱用户名或者
     * @return PageInfo
     */
    PageInfo<VppLicenseCodeVO> pageForReseller(Integer page, Integer pageSize, Integer productId, Integer status, String queryString);

    /**
     * 禁用某个序列码
     * @param licenseCodeId 序列码id
     */
    void disable(String licenseCodeId);

    /**
     * 获取用户对于某个产品的序列码
     * @param id 用户id
     * @param productCode 产品code
     */
    LicenseCodes getByMemberId(String id, String productCode);

    void insert(LicenseCodes licenseCodes);
}
