package cn.kdan.pdf.tech.core.service;

import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.params.VppMemberRegisterParam;
import cn.kdan.pdf.tech.core.params.VppMemberResetPwdParam;
import cn.kdan.pdf.tech.core.params.VppMemberUpdatePwdParam;
import cn.kdan.pdf.tech.core.pojo.oauth2.UserInfo;
import utils.BeanConverter;

import java.util.List;

/**
 * 用户service
 *
 * @author tangxiangan
 */
public interface VppMemberService extends BeanConverter<VppMember, UserInfo> {
    /**
     * 根据ID获取用户
     *
     * @param userId 用户ID
     * @return Members
     */
    VppMember getById(String userId);


    /**
     * 根据用户手机号和邮箱查找用户
     *
     * @param account 账号
     * @return Members
     */
    VppMember getByAccount(String account);

    /**
     * 获取sso用户
     * @param account account
     * @return
     */
    VppMember getByAccountSSO(String account);

    /**
     * 注册 超级管理员和管理员
     *
     * @param param param
     * @return VppMember
     */
    VppMember create(VppMemberRegisterParam param);

    /**
     * 试用管理员注册
     * @param param
     * @return
     */
    Boolean createTrial(VppMemberRegisterParam param);

    /**
     * 更新用户信息
     *
     * @param vppMember 用户
     */
    void insert(VppMember vppMember);

    /**
     * 更新用户信息
     *
     * @param vppMember 用户
     */
    void update(VppMember vppMember);

    void updateAll(VppMember vppMember);

    /**
     * 重置密码
     *
     * @param param param
     * @return boolean
     */
    boolean resetPassword(VppMemberResetPwdParam param);

    /**
     * 获取公司超管邮箱
     * @param companyId 公司id
     * @return
     */
    String getCompanySuperAdmin(String companyId);

    VppMember getSuperAdmin(String companyId);

    /**
     * 获取当前登录用户的公司所有用户，除超管
     * @param companyId
     * @return
     */
    List<VppMember> getComponyMenber(String companyId);

    /**
     * 修改密码
     *
     * @param param params
     * @return Boolean
     */
    boolean updatePassword(VppMemberUpdatePwdParam param);

    /**
     * 获取当前登录用户信息
     *
     * @return VppMember
     */
    VppMember getCurrentUser();

    /**
     * 判断用户是否存在
     *
     * @param account 邮箱账号
     * @return Boolean
     */
    Boolean checkExist(String account);

    /**
     * 修改用户角色
     *
     * @param memberId 邮箱账号
     * @param role     角色
     */
    void updateMemberRole(String memberId, String role);

    /**
     * 登出
     */
    void loginOut();

    /**
     * 删除目录下所有用户
     * @param directoryId 目录id
     */
    void deleteDirectoryMember(String directoryId);

    /**
     * 直接删除用户
     * @param memberId 用户id
     */
    void delete(String memberId);
}
