package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.OEMConstant;
import cn.kdan.pdf.tech.core.constant.VppDeviceConstant;
import cn.kdan.pdf.tech.core.enums.DeviceStatusEnum;
import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.enums.PlatformEnum;
import cn.kdan.pdf.tech.core.mapper.AppsMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.params.VerifyParam;
import cn.kdan.pdf.tech.core.pojo.oemVo.OemMemberVO;
import cn.kdan.pdf.tech.core.pojo.vo.ActivationVO;
import cn.kdan.pdf.tech.core.pojo.vo.DeviceResVO;
import cn.kdan.pdf.tech.core.pojo.vo.SubscriptionResVO;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.PeriodUtils;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;

import java.util.*;

@Slf4j
@Service
@Transactional
public class OEMServiceImpl implements OEMService {

    @Autowired
    private OemMemberService memberService;
    @Autowired
    private OemLicenseCodeService oemLicenseCodeService;
    @Autowired
    private ProductService productService;
    @Autowired
    private OemDeviceService oemDeviceService;
    @Autowired
    private AppsMapper appsMapper;
    @Autowired
    private OemLogService oemLogService;

    /**
     * 注册时增加序列码（增加订阅和订单）
     *
     * @param email
     * @param price
     * @return
     */
    @Override
    public OemMemberVO operateOEMUser(String email, String companyId, String productCode, Double price){
        Products product = productService.getByCode(productCode);
        //判断用户是否存在
        //如果用户不存在则增加用户信息
        OemMember member = memberService.getByEmail(email,companyId);
        String memberId;
        if(ObjectUtils.isEmpty(member)){
            //用户表增加数据
            OemMember members = new OemMember();
            memberId = CommonUtils.generateId();
            members.setId(memberId);
            members.setEmail(email);
            members.setCreatedAt(new Date());
            members.setUpdatedAt(new Date());
            members.setCompanyId(companyId);
            members.setValidFlag(ValidStatusEnum.VALID.value());
            memberService.insert(members);
        }else{
            memberId = member.getId();
        }
        //查询用户是否有code，如果有的话则增加时间，没有的话就新增
        OemLicenseCode licenseCode = oemLicenseCodeService.getByMemberId(memberId);
        if(ObjectUtils.isEmpty(licenseCode)){
            licenseCode = new OemLicenseCode();
            licenseCode.setId(CommonUtils.generateId());
            licenseCode.setMemberId(memberId);
            licenseCode.setTimes((short) 1);
            licenseCode.setUsedTimes((short) 0);
            licenseCode.setCdkey(generateCdKey());
            licenseCode.setCreatedAt(new Date());
            licenseCode.setUpdatedAt(new Date());
            licenseCode.setProductId(product.getId());
            licenseCode.setEndUpAt(PeriodUtils.countDate(new Date(), product.getForPeriod()));
            licenseCode.setType((short) (product.getCode().contains("trail")? 2:1));
            licenseCode.setValidFlag(LicenseCodeStatusEnum.ASSIGNED.code());
            oemLicenseCodeService.insert(licenseCode);
        }else{
            //如果大于当前时间
            if(licenseCode.getEndUpAt().after(new Date())){
                licenseCode.setEndUpAt(PeriodUtils.countDate(licenseCode.getEndUpAt(), product.getForPeriod()));
            } else{
                licenseCode.setEndUpAt(PeriodUtils.countDate(new Date(), product.getForPeriod()));
            }
            licenseCode.setProductId(product.getId());
            licenseCode.setValidFlag(LicenseCodeStatusEnum.ASSIGNED.code());
            licenseCode.setUpdatedAt(new Date());
            licenseCode.setType((short) (product.getCode().contains("Trail")? 2:1));
            oemLicenseCodeService.update(licenseCode);
        }
        OemMemberVO oemMemberVO =  new OemMemberVO();
        oemMemberVO.setEmail(email);
        oemMemberVO.setLicenseCode(licenseCode.getCdkey());
        OemLog oemLog = new OemLog();
        oemLog.setCompanyId(companyId);
        oemLog.setId(CommonUtils.generateId());
        oemLog.setCreatedAt(new Date());
        oemLog.setUpdatedAt(new Date());
        oemLog.setMemberId(memberId);
        oemLog.setProductId(product.getId());
        oemLog.setOemLicenseCodeId(licenseCode.getId());
        oemLog.setValidFlag(ValidStatusEnum.VALID.value());
        oemLog.setOemPrice(price);
        oemLogService.insert(oemLog);
        return oemMemberVO;
    }

    @Override
    public List<String> generateCode(String companyId, String productCode, Integer number){
        // 获取产品信息
        Products product = productService.getByCode(productCode);

        // 创建一个列表来存储生成的cdkey
        List<String> cdkeys = new ArrayList<>();

        for (int i = 0; i < number; i++) {
            // 创建新的OemLicenseCode实例
            OemLicenseCode licenseCode = new OemLicenseCode();
            licenseCode.setId(CommonUtils.generateId());
            licenseCode.setTimes((short) 1);
            licenseCode.setUsedTimes((short) 0);
            licenseCode.setCdkey(generateCdKey());
            licenseCode.setCreatedAt(new Date());
            licenseCode.setUpdatedAt(new Date());
            licenseCode.setProductId(product.getId());
            licenseCode.setEndUpAt(PeriodUtils.countDate(new Date(), product.getForPeriod()));
            licenseCode.setType((short) (product.getCode().contains("trail") ? 2 : 1));
            licenseCode.setValidFlag(LicenseCodeStatusEnum.ASSIGNED.code());
            licenseCode.setCompanyId(companyId);
            // 将OemLicenseCode实例插入数据库
            oemLicenseCodeService.insert(licenseCode);

            // 将生成的cdkey添加到列表中
            cdkeys.add(licenseCode.getCdkey());
        }

        // 返回包含所有cdkey的列表
        return cdkeys;
    }

    @Override
    public ActivationVO activation(VerifyParam verifyParam) {
        ActivationVO activationVO = new ActivationVO();
        // 查询序列码信息
        OemLicenseCode licenseCodesVO = oemLicenseCodeService.getByCdKey(verifyParam.getSubscription().getCdkey());
        if (ObjectUtils.isEmpty(licenseCodesVO)) {
            throw new BackendRuntimeException(OEMConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
        }

        // 验证序列码使用次数是否超过限制
        if (licenseCodesVO.getUsedTimes() >= licenseCodesVO.getTimes()) {
            throw new BackendRuntimeException(OEMConstant.EXCEPTION_MSG_MAX_DEVICE_NUM);
        }

        // 验证序列码是否已过期
        if (licenseCodesVO.getEndUpAt().before(new Date())) {
            throw new BackendRuntimeException(OEMConstant.EXCEPTION_MSG_LICENSE_EXPIRED);
        }

        // 验证序列码的有效标志
        if (!licenseCodesVO.getValidFlag().equals(LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code())
                && !licenseCodesVO.getValidFlag().equals(LicenseCodeStatusEnum.ASSIGNED.code())) {
            throw new BackendRuntimeException(OEMConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
        }

        Products products = productService.getById(licenseCodesVO.getProductId());

        OemDevice oemDevice  = oemDeviceService.getDeviceByUniqueSn(verifyParam.getDevice().getUniqueSn(),products.getAppId());
        if (ObjectUtils.isEmpty(oemDevice)) {
            oemDevice = new OemDevice();
            // 新增设备
            oemDevice.setCompanyId(licenseCodesVO.getCompanyId());
            oemDevice.setId(UUID.randomUUID().toString());
            oemDevice.setAppId(products.getAppId());
            oemDevice.setEndUpAt(licenseCodesVO.getEndUpAt());
            if(!StringUtils.isEmpty(licenseCodesVO.getMemberId())){
                oemDevice.setMemberId(licenseCodesVO.getMemberId());
            }
            PlatformEnum platformEnum = PlatformEnum.getEnumByCode(verifyParam.getDevice().getPlatform());
            oemDevice.setPlatform(platformEnum.code());
            oemDevice.setStatus(licenseCodesVO.getType().equals(Short.valueOf("1"))?DeviceStatusEnum.ACTIVE.code():DeviceStatusEnum.ON_TRIAL.code());
            oemDevice.setCdkey(licenseCodesVO.getCdkey());
            oemDevice.setUniqueSn(verifyParam.getDevice().getUniqueSn());
            oemDevice.setActivatedDate(new Date());
            oemDevice.setCreatedAt(new Date());
            oemDevice.setUpdatedAt(new Date());
            oemDeviceService.insert(oemDevice);
        }else {
            if (licenseCodesVO.getValidFlag().equals(LicenseCodeStatusEnum.ACTIVE.code())){
                // 设备已激活，返回设备信息
                DeviceResVO deviceResVO = new DeviceResVO();
                deviceResVO.setStatus(Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(oemDevice.getStatus())).value());
                deviceResVO.setProduct_code(products.getCode());
                activationVO.setDevice(deviceResVO);
                return activationVO;
            }else{
                oemDevice.setEndUpAt(licenseCodesVO.getEndUpAt());
                if(!StringUtils.isEmpty(licenseCodesVO.getMemberId())){
                    oemDevice.setMemberId(licenseCodesVO.getMemberId());
                }
                oemDevice.setStatus(licenseCodesVO.getType().equals(Short.valueOf("1"))?DeviceStatusEnum.ACTIVE.code():DeviceStatusEnum.ON_TRIAL.code());
                oemDevice.setCdkey(licenseCodesVO.getCdkey());
                oemDevice.setActivatedDate(new Date());
                oemDevice.setUpdatedAt(new Date());
                oemDeviceService.updateSelective(oemDevice);
            }
        }

        // 更新序列码的使用次数和有效标志
        OemLicenseCode updateCode = new OemLicenseCode();
        updateCode.setId(licenseCodesVO.getId());
        updateCode.setUsedTimes((short) (licenseCodesVO.getUsedTimes() + 1));
        if (updateCode.getUsedTimes() < licenseCodesVO.getTimes()) {
            updateCode.setValidFlag(LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code());
        }
        if (updateCode.getUsedTimes().equals(licenseCodesVO.getTimes())) {
            updateCode.setValidFlag(LicenseCodeStatusEnum.ACTIVE.code());
        }
        oemLicenseCodeService.update(updateCode);

        // 返回设备信息
        DeviceResVO deviceResVO = new DeviceResVO();
        deviceResVO.setStatus((Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(oemDevice.getStatus()))).value());
        deviceResVO.setProduct_code(products.getCode());
        deviceResVO.setUniqueSn(oemDevice.getUniqueSn());
        activationVO.setDevice(deviceResVO);
        return activationVO;
    }


    @Override
    public ActivationVO verify(VerifyParam verifyParam) {
        Integer appID = getAppID(verifyParam);
        // 查询设备是否已激活
        OemDevice devices = oemDeviceService.getDeviceByUniqueSn(verifyParam.getDevice().getUniqueSn(),appID);
        DeviceResVO deviceResVO = new DeviceResVO();
        if(!ObjectUtils.isEmpty(devices)){
            devices.setAppVersion(verifyParam.getDevice().getAppVersion());
            oemDeviceService.updateSelective(devices);
        }else{
            devices = new OemDevice();
            // 新增设备
            devices.setAppId(appID);
            devices.setId(UUID.randomUUID().toString());
            devices.setAppVersion(verifyParam.getDevice().getAppVersion());
            devices.setModel(verifyParam.getDevice().getModel());
            devices.setOs(verifyParam.getDevice().getOs());
            devices.setUniqueSn(verifyParam.getDevice().getUniqueSn());
            devices.setTimeZone(verifyParam.getDevice().getTimeZone());
            PlatformEnum platformEnum = PlatformEnum.getEnumByCode(verifyParam.getDevice().getPlatform());
            devices.setPlatform(platformEnum.code());
            devices.setStatus(DeviceStatusEnum.UNACTIVATED.code());
            devices.setCreatedAt(new Date());
            devices.setUpdatedAt(new Date());
            oemDeviceService.insert(devices);
        }
        SubscriptionResVO subscription = new SubscriptionResVO();
        deviceResVO.setUniqueSn(devices.getUniqueSn());
        deviceResVO.setStatus(Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(devices.getStatus())).value());
        if(!ObjectUtils.isEmpty(devices)&&(devices.getStatus().equals(DeviceStatusEnum.ACTIVE.code())||devices.getStatus().equals(DeviceStatusEnum.ON_TRIAL.code()))){
            OemLicenseCode licenseCodesVO = oemLicenseCodeService.getByCdKey(devices.getCdkey());
            if (ObjectUtils.isEmpty(licenseCodesVO)) {
                throw new BackendRuntimeException(OEMConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
            }
            Products products = productService.getById(licenseCodesVO.getProductId());
            subscription.setStartDate(licenseCodesVO.getCreatedAt().getTime()/1000);
            subscription.setEndDate(licenseCodesVO.getEndUpAt().getTime()/1000);
            deviceResVO.setProduct_code(products.getCode());
            deviceResVO.setProduct_code(products.getCode());
            deviceResVO.setCdkey(licenseCodesVO.getCdkey());
        }
        ActivationVO activationVO = new ActivationVO();
        activationVO.setDevice(deviceResVO);
        activationVO.setSubscription(subscription);
        return activationVO;
    }

    private Integer getAppID(VerifyParam verifyParam) {
        AppsExample example = new AppsExample();
        example.createCriteria().andCodeEqualTo(verifyParam.getSubscription().getAppCode());
        List<Apps> apps = appsMapper.selectByExample(example);
        Integer appID = null;
        if (!CollectionUtils.isEmpty(apps)) {
            appID = apps.get(0).getId();
        }
        return appID;
    }

    @Override
    public void deleteOEMUser(String email, String companyId) {
        //判断用户是否存在
        //如果用户不存在则增加用户信息
        OemMember member = memberService.getByEmail(email,companyId);
        member.setValidFlag(ValidStatusEnum.INVALID.value());
        memberService.update(member);
        OemLicenseCode licenseCodes = oemLicenseCodeService.getByMemberId(member.getId());
        licenseCodes.setValidFlag(LicenseCodeStatusEnum.DELETED.code());
        oemDeviceService.invalidateDevice(licenseCodes.getCdkey());
    }

    @Override
    public void cancel(VerifyParam verifyParam) {
        Integer appID = getAppID(verifyParam);
        OemDevice devices = oemDeviceService.getDeviceByUniqueSn(verifyParam.getDevice().getUniqueSn(),appID);
        if (ObjectUtils.isEmpty(devices)) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_DEVICE_NOT_EXIST);
        }

        //设置序列码状态
        String cdKey = devices.getCdkey();
        OemLicenseCode code = oemLicenseCodeService.getByCdKey(cdKey);

        //更新设备状态
        devices.setStatus(DeviceStatusEnum.UNACTIVATED.code());
        devices.setCanceledDate(new Date());
        devices.setEndUpAt(null);
        devices.setCdkey(null);
        log.info("update device:" + devices);
        oemDeviceService.update(devices);
        // 获取序列码下所有已激活设备
        long countByExample = oemDeviceService.getUseDeviceCount(cdKey);
        code.setUsedTimes((short) countByExample);
        Integer code1 = countByExample == 0 ? LicenseCodeStatusEnum.ASSIGNED.code() : LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code();
        code.setValidFlag(code1);
        code.setUpdatedAt(new Date());
        log.info("update code:" + code);
        oemLicenseCodeService.update(code);
    }

    //购买产品更新会员

    public String generateCdKey() {
        String characters = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        Random random = new Random();
        String cdKey;
        int attempts = 0;
        OemLicenseCode codes = new OemLicenseCode();
        do {
            StringBuilder sb = new StringBuilder();
            // 循环生成16位随机字符，并在每4位后添加一个连字符
            for (int i = 0; i < 16; i++) {
                if (i > 0 && i % 4 == 0) {
                    sb.append("-");
                }
                int index = random.nextInt(characters.length());
                sb.append(characters.charAt(index));
            }
            cdKey = sb.toString();

            // 检查生成的CD-Key是否已经存在
            codes = oemLicenseCodeService.getByCdKey(cdKey);
            attempts++;
            if (attempts >= 3) {
                // 如果达到最大尝试次数，抛出异常或返回错误信息
                throw new RuntimeException("Unable to generate a unique CD-Key after " + 3 + " attempts.");
            }
        } while (codes != null); // 如果存在，继续尝试

        // 如果不存在，返回生成的CD-Key
        return cdKey;
    }
}
