package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.DeviceStatusEnum;
import cn.kdan.pdf.tech.core.mapper.OemDeviceMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Slf4j
@Service
@Transactional
public class OemDeviceServiceImpl implements OemDeviceService {

    @Autowired
    private OemDeviceMapper deviceMapper;


    @Override
    public OemDevice getDeviceByUniqueSn(String uniqueSn, Integer appId) {
        // 定义设备状态集合，用于判断设备是否已激活
        Set<Integer> allStatuses = new HashSet<>();
        allStatuses.add(DeviceStatusEnum.ACTIVE.code());
        allStatuses.add(DeviceStatusEnum.UNACTIVATED.code());
        allStatuses.add(DeviceStatusEnum.ON_TRIAL.code());
        allStatuses.add(DeviceStatusEnum.EXPIRED_ON_TRIAL.code());
        OemDeviceExample devicesExample = new OemDeviceExample();
        devicesExample.createCriteria()
                .andUniqueSnEqualTo(uniqueSn)
                .andAppIdEqualTo(appId)
                .andStatusIn(new ArrayList<>(allStatuses));
        List<OemDevice> list = deviceMapper.selectByExample(devicesExample);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    public void insert(OemDevice device) {
        deviceMapper.insert(device);
    }

    @Override
    public void updateSelective(OemDevice device) {
        deviceMapper.updateByPrimaryKeySelective(device);
    }

    @Override
    public void update(OemDevice device) {
        deviceMapper.updateByPrimaryKey(device);
    }

    @Override
    public void expireDevice(String cdKey,DeviceStatusEnum status) {
        OemDeviceExample devicesExample = new OemDeviceExample();
        devicesExample.createCriteria()
                .andCdkeyEqualTo(cdKey);
        OemDevice device = new OemDevice();
        device.setStatus(DeviceStatusEnum.EXPIRED.code());
        device.setUpdatedAt(new Date());
        deviceMapper.updateByExampleSelective(device,devicesExample);
    }

    @Override
    public void invalidateDevice(String cdKey) {
        OemDeviceExample devicesExample = new OemDeviceExample();
        devicesExample.createCriteria()
                .andCdkeyEqualTo(cdKey);
        OemDevice device = new OemDevice();
        device.setStatus(DeviceStatusEnum.DELETED.code());
        device.setUpdatedAt(new Date());
        deviceMapper.updateByExampleSelective(device,devicesExample);
    }

    @Override
    public long getUseDeviceCount(String cdKey) {
        OemDeviceExample devicesExample = new OemDeviceExample();
        devicesExample.createCriteria().andCdkeyEqualTo(cdKey).andStatusEqualTo(DeviceStatusEnum.ACTIVE.code());
        return deviceMapper.countByExample(devicesExample);
    }
}
