package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.mapper.OemLicenseCodeMapper;
import cn.kdan.pdf.tech.core.model.LicenseCodes;
import cn.kdan.pdf.tech.core.model.OemLicenseCode;
import cn.kdan.pdf.tech.core.model.OemLicenseCodeExample;
import cn.kdan.pdf.tech.core.service.OemLicenseCodeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class OemLicenseCodeServiceImpl implements OemLicenseCodeService {

    @Autowired
    private OemLicenseCodeMapper oemLicenseCodeMapper;


    @Override
    public void insert(OemLicenseCode oemLicenseCode){
        oemLicenseCode.setCreatedAt(new Date());
        oemLicenseCode.setUpdatedAt(new Date());
        oemLicenseCodeMapper.insert(oemLicenseCode);
    }

    @Override
    public OemLicenseCode getByMemberId(String memberId){
        OemLicenseCodeExample example = new OemLicenseCodeExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        List<OemLicenseCode> oemLicenseCodes = oemLicenseCodeMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(oemLicenseCodes)){
            return null;
        }
        return oemLicenseCodes.get(0);
    }

    @Override
    public void update(OemLicenseCode licenseCode) {
        oemLicenseCodeMapper.updateByPrimaryKeySelective(licenseCode);
    }

    @Override
    public OemLicenseCode getByCdKey(String cdKey) {
        OemLicenseCodeExample example = new OemLicenseCodeExample();
        OemLicenseCodeExample.Criteria criteria = example.createCriteria();
        criteria.andCdkeyEqualTo(cdKey);
        List<OemLicenseCode> list = oemLicenseCodeMapper.selectByExample(example);
        return CollectionUtils.isEmpty(list) ?  null: list.get(0) ;
    }

    @Override
    public List<OemLicenseCode> listActive() {
        List<Integer> values = new ArrayList<>();
        values.add(LicenseCodeStatusEnum.DELETED.code());
        values.add(LicenseCodeStatusEnum.EXPIRED.code());
        values.add(LicenseCodeStatusEnum.Refunded.code());
        OemLicenseCodeExample example = new OemLicenseCodeExample();
        example.createCriteria().andValidFlagNotIn(values);
        return oemLicenseCodeMapper.selectByExample(example);
    }

}
