package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.mapper.ext.ExtProductMapper;
import cn.kdan.pdf.tech.core.model.Products;
import cn.kdan.pdf.tech.core.model.ProductsExample;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.pojo.vo.ProductListVO;
import cn.kdan.pdf.tech.core.pojo.vo.ProductSimpleVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppRTeamMemberRoleVO;
import cn.kdan.pdf.tech.core.service.ProductService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import cn.kdan.pdf.tech.core.service.VppRTeamMemberRoleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * oauth2 用户方法类
 *
 * @author tangxiangan
 */
@Slf4j
@Service
public class ProductServiceImpl implements ProductService {

    @Resource
    private ExtProductMapper productMapper;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Resource
    private VppMemberService vppMemberService;

    @Override
    public List<ProductSimpleVO> listWithSimpleInfo(String teamId) {
        return productMapper.listWithSimpleInfo(teamId);
    }

    @Override
    public Products getById(Integer productId) {
        return productMapper.selectByPrimaryKey(productId);
    }

    public Products get(Integer productId) {
        return productMapper.selectByPrimaryKey(productId);
    }


    @Override
    public Products getByName(String productName) {
        ProductsExample productsExample = new ProductsExample();
        productsExample.createCriteria().andNameEqualTo(productName);
        List<Products> products = productMapper.selectByExample(productsExample);
        return CollectionUtils.isEmpty(products) ? null : products.get(0);
    }

    @Override
    public List<Products> getByCodeList(List<String> codeList) {
        ProductsExample productsExample = new ProductsExample();
        productsExample.createCriteria().andCodeIn(codeList);
        return productMapper.selectByExample(productsExample);
    }


    @Override
    public List<ProductListVO> listWithAdmin() {
        VppMember member = vppMemberService.getCurrentUser();
        List<VppRTeamMemberRoleVO> list = vppRTeamMemberRoleService.listByMemberIdAdmin(member.getId());
        List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
        idList = CollectionUtils.isEmpty(idList)?null:idList;
        return productMapper.listWithAdmin(idList);
    }

    @Override
    public List<ProductListVO> listWithAll() {
        VppMember member = vppMemberService.getCurrentUser();
        return productMapper.listWithAll(member.getCompanyId());
    }

    @Override
    public Products getByCode(String productCode) {
        ProductsExample productsExample = new ProductsExample();
        productsExample.createCriteria().andCodeEqualTo(productCode);
        List<Products> products = productMapper.selectByExample(productsExample);
        return  CollectionUtils.isEmpty(products) ? null : products.get(0);
    }

    @Override
    public List<ProductListVO> purchasedProductList() {
        VppMember member = vppMemberService.getCurrentUser();
        List<ProductListVO> allList = productMapper.groupListWithCompanyAndStatus(member.getCompanyId(), null);

        if (CollectionUtils.isEmpty(allList)) {
            return new ArrayList<>();
        }

        List<ProductListVO> assignList = productMapper.groupListWithCompanyAndStatus(member.getCompanyId(), LicenseCodeStatusEnum.ASSIGNED.code());

        Map<Integer, Integer> assignedProductMap = assignList.stream()
                .collect(Collectors.toMap(ProductListVO::getId, ProductListVO::getCodeNumber));

       // 遍历所有产品列表，设置 assignedCodeNumber
        allList.forEach(productListVO -> {
            Integer assignedCodeNumber = assignedProductMap.get(productListVO.getId());
            productListVO.setAssignedCodeNumber(assignedCodeNumber != null ? assignedCodeNumber : 0);
        });

        return allList;
    }
}
