package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.ScheduleSettingTypeEnum;
import cn.kdan.pdf.tech.core.mapper.ScheduleSettingMapper;
import cn.kdan.pdf.tech.core.model.ScheduleSetting;
import cn.kdan.pdf.tech.core.model.ScheduleSettingExample;
import cn.kdan.pdf.tech.core.pojo.dto.ScheduleSettingDTO;
import cn.kdan.pdf.tech.core.service.ScheduleSettingService;
import cn.kdan.pdf.tech.core.service.SyncService;
import cn.kdan.pdf.tech.core.utils.QuartzUtils;
import cn.kdan.pdf.tech.core.utils.SyncJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import utils.CommonUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class ScheduleSettingServiceImpl implements ScheduleSettingService {
    @Resource
    private ScheduleSettingMapper scheduleSettingMapper;
    @Resource
    private SyncService syncService;
    @Override
    public List<ScheduleSetting> listByDirectoryId(String directoryId) {
        ScheduleSettingExample scheduleSettingExample = new ScheduleSettingExample();
        scheduleSettingExample.createCriteria().andDirectoryIdEqualTo(directoryId);
        List<ScheduleSetting> list = scheduleSettingMapper.selectByExample(scheduleSettingExample);
        return CollectionUtils.isEmpty(list) ? new ArrayList<>() : list;
    }

    @Override
    public List<ScheduleSetting> list() {
        ScheduleSettingExample scheduleSettingExample = new ScheduleSettingExample();
        scheduleSettingExample.createCriteria().andTypeNotEqualTo(ScheduleSettingTypeEnum.SYNC_NOW.value());
        List<ScheduleSetting> list = scheduleSettingMapper.selectByExample(scheduleSettingExample);
        return CollectionUtils.isEmpty(list) ? new ArrayList<>() : list;
    }

    /**
     * 同步设置
     * @param scheduleSettingDTO scheduleSettingDTO
     */
    @Override
    public void syncSetting(ScheduleSettingDTO scheduleSettingDTO) {
        List<ScheduleSetting> list = listByDirectoryId(scheduleSettingDTO.getDirectoryId());
        ScheduleSetting scheduleSetting = new ScheduleSetting();
        BeanUtils.copyProperties(scheduleSettingDTO, scheduleSetting);
        if(scheduleSettingDTO.getType().equals(ScheduleSettingTypeEnum.SYNC_NOW.value())){
            syncService.startSync(scheduleSetting.getDirectoryId());
            //如果之前不为空，现在是立即同步，那么要把以前的定时任务给删掉
            if(!CollectionUtils.isEmpty(list)){
                QuartzUtils.deleteJob("syncJob" + list.get(0).getScheduleSettingId(),"syncJobGroup");
                scheduleSettingMapper.deleteByPrimaryKey(list.get(0).getScheduleSettingId());
            }
        }else {
            String cronExpression = "";
            if(scheduleSettingDTO.getType().equals(ScheduleSettingTypeEnum.PER_HOUR.value())){
                cronExpression = "0 0 * * * ?";
            }
            if(scheduleSettingDTO.getType().equals(ScheduleSettingTypeEnum.SPECIFIED_TIME_EVERY_DAY.value())){
                cronExpression = QuartzUtils.generateCronExpression(scheduleSetting.getTime());
            }
            //先判断是否已有配置
            if(CollectionUtils.isEmpty(list)){
                scheduleSetting.setScheduleSettingId(CommonUtils.generateId());
                scheduleSetting.setType(scheduleSetting.getType());
                if(scheduleSettingDTO.getType().equals(ScheduleSettingTypeEnum.SPECIFIED_TIME_EVERY_DAY.value())){
                    scheduleSetting.setTime(scheduleSettingDTO.getTime());
                }
                scheduleSettingMapper.insertSelective(scheduleSetting);
                //创建定时任务
                QuartzUtils.scheduleJob("syncJob" + scheduleSetting.getScheduleSettingId(),"syncJobGroup", SyncJob.class,
                        "syncTrigger_" + scheduleSetting.getScheduleSettingId(),"syncTriggerGroup",
                        cronExpression,scheduleSetting.getDirectoryId() );
            }else {
                list.get(0).setTime(scheduleSettingDTO.getTime());
                list.get(0).setType(scheduleSettingDTO.getType());
                scheduleSettingMapper.updateByPrimaryKeySelective(list.get(0));
                QuartzUtils.rescheduleJob("syncTrigger_" + scheduleSetting.getScheduleSettingId(), "syncTriggerGroup", cronExpression,scheduleSetting);
            }
        }
    }

    @Override
    public void deleteByDirectoryId(String directoryId) {
        List<ScheduleSetting> list = listByDirectoryId(directoryId);
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        QuartzUtils.deleteJob("syncJob" + list.get(0).getScheduleSettingId(),"syncJobGroup");
        ScheduleSettingExample scheduleSettingExample = new ScheduleSettingExample();
        scheduleSettingExample.createCriteria().andDirectoryIdEqualTo(directoryId);
        scheduleSettingMapper.deleteByExample(scheduleSettingExample);
    }

    @PostConstruct
    public void initJob() {
        log.info("initJob start:--------------");
        List<ScheduleSetting> list = list();
        for (ScheduleSetting scheduleSetting : list) {
            if (scheduleSetting.getType().equals(ScheduleSettingTypeEnum.PER_HOUR.value())) {
                //创建定时任务
                QuartzUtils.scheduleJob("syncJob" + scheduleSetting.getScheduleSettingId(), "syncJobGroup", SyncJob.class,
                        "syncTrigger_" + scheduleSetting.getScheduleSettingId(), "syncTriggerGroup",
                        "0 0 * * * ?", scheduleSetting.getDirectoryId());
            } else if (scheduleSetting.getType().equals(ScheduleSettingTypeEnum.SPECIFIED_TIME_EVERY_DAY.value())) {
                //创建定时任务
                QuartzUtils.scheduleJob("syncJob" + scheduleSetting.getScheduleSettingId(), "syncJobGroup", SyncJob.class,
                        "syncTrigger_" + scheduleSetting.getScheduleSettingId(), "syncTriggerGroup",
                        QuartzUtils.generateCronExpression(scheduleSetting.getTime()), scheduleSetting.getDirectoryId());
            }
        }
        log.info("initJob finish:--------------");
    }

}
