package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.SSOConstant;
import cn.kdan.pdf.tech.core.constant.VppTeamConstant;
import cn.kdan.pdf.tech.core.enums.*;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.model.Directory;
import cn.kdan.pdf.tech.core.model.Domain;
import cn.kdan.pdf.tech.core.pojo.MSGroup;
import cn.kdan.pdf.tech.core.pojo.dto.AddSyncDTO;
import cn.kdan.pdf.tech.core.pojo.dto.CreateTeamDTO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMemberVO;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.QuartzUtils;
import cn.kdan.pdf.tech.core.utils.SyncJob;
import cn.kdan.pdf.tech.core.utils.ThreadPoolSingleUtil;
import com.azure.identity.ClientSecretCredential;
import com.azure.identity.ClientSecretCredentialBuilder;
import com.microsoft.graph.models.*;
import com.microsoft.graph.serviceclient.GraphServiceClient;

import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@Transactional
public class SyncServiceImpl implements SyncService {
    @Resource
    private DirectoryService directoryService;
    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppTeamService teamService;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Resource
    private DirectorySyncService directorySyncService;
    @Resource
    private DomainService domainService;
    @Override
    public void addSyncSettings(AddSyncDTO syncDTO) {
        //根据id获取目录
        Directory directory = directoryService.getById(syncDTO.getDirectoryId());
        //修改数据库
        directory.setApplicationId(syncDTO.getApplicationID());
        directory.setClientSecrets(syncDTO.getClientSecrets());
        //
        directory.setScope(syncDTO.getScope());
        directory.setTenantId(syncDTO.getTenantId());
        directory.setUpdateTime(new Date());
        if (!CollectionUtils.isEmpty(syncDTO.getGroupIds())) {
            directory.setMsGroupIds(listToString(syncDTO.getGroupIds()));
        }
        directoryService.updateSelective(directory);
        //创建一个目录的默认团队，判断是否有默认团队了
        String teamId = directory.getTeamId();
        VppTeam team = teamService.getById(teamId);
        if (ObjectUtils.isEmpty(team)) {
            //没有团队则先创建默认团队
            teamId = CommonUtils.generateId();
            CreateTeamDTO dto = new CreateTeamDTO();
            dto.setTeamName(directory.getName());
            dto.setTeamDescription("sync");
            dto.setProductList(syncDTO.getProductList());
            dto.setTeamId(teamId);
            dto.setType(3);
            //创建团队
            teamService.createTeam(dto);
            //绑定目录和团队
            directory.setTeamId(teamId);
            directoryService.updateSelective(directory);
        }
        ThreadPoolSingleUtil.getInstance().excute(() -> {
            sync(syncDTO.getScope(), directory);
        });
    }

    private void sync(String scope, Directory directory) {
        String syncId = startLog(directory);
        try {
        if (scope.equals(ScpoeEnum.ALL.value())) {
            //同步全部
            syncMSUserAll(directory);
        }
        if (scope.equals(ScpoeEnum.SPECIFY.value())) {
            //同步特定
            syncMSUserSpecifyGroup(directory);
        }
            endLog(syncId,SyncStatusEnum.SYNC_FINISHED);
        } catch (Exception e) {
            endLog(syncId,SyncStatusEnum.SYNC_FAILED);
            log.error("SYNC_FAILED:{}", e.getMessage());
        }
    }

    @Override
    public void startSync(String directoryId) {
        //根据id获取目录
        Directory directory = directoryService.getById(directoryId);
        //修改数据库
        directory.setApplicationId(directory.getApplicationId());
        directory.setClientSecrets(directory.getClientSecrets());
        directory.setScope(directory.getScope());
        directory.setTenantId(directory.getTenantId());
        directory.setUpdateTime(new Date());
        directoryService.updateSelective(directory);
        //创建一个目录的默认团队，判断是否有默认团队了
        String teamId = directory.getTeamId();
        VppTeam team = teamService.getById(teamId);
        if (ObjectUtils.isEmpty(team)) {
            log.error("TEAM_NOT_EXIST:{}", teamId);
            throw new RuntimeException(VppTeamConstant.TEAM_NOT_EXIST);
        }
        ThreadPoolSingleUtil.getInstance().excute(() -> {
            sync(directory.getScope(), directory);
        });
    }

    /**
     * 同步MS用户同步所有
     *
     * @param directory 目录对象
     */
    public void syncMSUserAll(Directory directory){
        String companyId = directory.getCompanyId();
        String teamId = directory.getTeamId();
        final String[] scopes = new String[]{SSOConstant.GRAPH_SCOPE};
        final ClientSecretCredential credential = new ClientSecretCredentialBuilder()
                .clientId(directory.getApplicationId())
                .tenantId(directory.getTenantId())
                .clientSecret(directory.getClientSecrets())
                .build();
        final GraphServiceClient graphClient = new GraphServiceClient(credential, scopes);
        checkDomain(directory, graphClient);
        UserCollectionResponse userResponse = graphClient.users().get();
        List<User> users = userResponse.getValue();
        List<String> microsoftEmails = users.stream().map(User::getUserPrincipalName).collect(Collectors.toList());

        handleUsersData(users, microsoftEmails, companyId, teamId, directory.getDirectoryId());
    }

    private void endLog(String syncId,SyncStatusEnum syncStatusEnum) {
        DirectorySync directorySync = new DirectorySync();
        directorySync.setStatus(syncStatusEnum.value());
        directorySync.setSyncTime(new Date());
        directorySync.setDirectorySyncId(syncId);
        directorySyncService.update(directorySync);
    }

    private String startLog(Directory directory) {
        DirectorySync directorySync = new DirectorySync();
        String syncId =  CommonUtils.generateId();
        directorySync.setDirectoryId(directory.getDirectoryId());
        directorySync.setStatus(SyncStatusEnum.SYNC_ING.value());
        directorySync.setSyncTime(new Date());
        directorySync.setDirectorySyncId(syncId);
        directorySyncService.create(directorySync);
        return syncId;
    }

    /**
     * 同步MS用户同步特定
     *
     * @param directory 目录对象
     */
    public void syncMSUserSpecifyGroup(Directory directory){
        String companyId = directory.getCompanyId();
        String teamId = directory.getTeamId();
        List<String> groupIds = stringToList(directory.getMsGroupIds());
        final String[] scopes = new String[]{SSOConstant.GRAPH_SCOPE};
        final ClientSecretCredential credential = new ClientSecretCredentialBuilder()
                .clientId(directory.getApplicationId())
                .tenantId(directory.getTenantId())
                .clientSecret(directory.getClientSecrets())
                .build();
        final GraphServiceClient graphClient = new GraphServiceClient(credential, scopes);
        checkDomain(directory, graphClient);
        List<User> users = new ArrayList<>();
        groupIds.forEach(groupId -> {
            Group group = graphClient.groups().byGroupId(groupId).get(requestConfiguration -> {
                requestConfiguration.queryParameters.expand = new String[]{"members"};
            });
            group.getMembers().forEach(member -> {
                if (member instanceof User) {
                    users.add((User) member);
                }
            });
        });

        List<String> microsoftEmails = users.stream().map(User::getUserPrincipalName).collect(Collectors.toList());

        handleUsersData(users, microsoftEmails, companyId, teamId, directory.getDirectoryId());
    }

    private void checkDomain(Directory directory, GraphServiceClient graphClient) {
        List<Domain> list = domainService.listByDirectoryId(directory.getDirectoryId());
        if(CollectionUtils.isEmpty(list)){
            throw new BackendRuntimeException(SSOConstant.DOMAIN_NOT_EXIST);
        }
        try {
            DomainCollectionResponse domainCollectionResponse =  graphClient.domains().get();
            List<String> domainList = domainCollectionResponse.getValue()
                    .stream()
                    .map(domain -> domain.getId().toLowerCase()) // 将ID转换为小写
                    .collect(Collectors.toList());
            if(!domainList.contains(list.get(0).getDomain().toLowerCase())){
                throw new BackendRuntimeException(SSOConstant.DOMAIN_NOT_EXIST);
            }
        }catch (Exception e){
            throw new BackendRuntimeException(SSOConstant.DOMAIN_NOT_EXIST);
        }
    }

    /**
     * 对比微软用户和我们指定组用户的差别，进行同步
     *
     * @param users           users
     * @param microsoftEmails microsoftEmails
     * @param companyId       companyId
     * @param teamId          teamId
     * @param directoryId
     */
    private void handleUsersData(List<User> users, List<String> microsoftEmails, String companyId, String teamId, String directoryId) {
        for (User user : users) {
            String email = user.getUserPrincipalName();
            String name = user.getDisplayName();
            String id = user.getId();
            VppMember member = vppMemberService.getByAccountSSO(email);

            if (ObjectUtils.isEmpty(member)) {
                VppMember newMember = new VppMember().withId(id)
                        .withFullName(name)
                        .withEmail(email)
                        .withValidFlag(ValidStatusEnum.VALID.value())
                        .withCreatedAt(new Date())
                        .withUpdatedAt(new Date())
                        .withSubscriberType(0)
                        .withRole(RoleEnum.TEAM_MEMBER.code())
                        .withIsSso(SyncUserEnum.SSO.value())
                        .withCompanyId(companyId)
                        .withDirectoryId(directoryId);
                vppMemberService.insert(newMember);

                VppRTeamMemberRole vppRTeamMemberRole = new VppRTeamMemberRole()
                        .withMemberId(id)
                        .withRoleId(RoleEnum.TEAM_MEMBER.code())
                        .withTeamId(teamId)
                        .withId(CommonUtils.generateId());
                vppRTeamMemberRoleService.create(vppRTeamMemberRole);
            } else {
                member.setFullName(name);
                vppMemberService.update(member);
            }
        }

        List<VppTeamMemberVO> localMembers = vppRTeamMemberRoleService.listForMember(teamId, null, null, null, "1", null);
        localMembers.stream()
                .filter(member -> !microsoftEmails.contains(member.getEmail()))
                .forEach(member -> vppRTeamMemberRoleService.deleteMember(member.getId()));
    }

    @Override
    public List<MSGroup> getGroupList(String clientId, String tenantId, String clientSecret, String queryString) {
        final String[] scopes = new String[] { "https://graph.microsoft.com/.default" };
        ClientSecretCredential credential = new ClientSecretCredentialBuilder()
                .clientId(clientId).tenantId(tenantId).clientSecret(clientSecret).build();
        GraphServiceClient graphClient = new GraphServiceClient(credential, scopes);
        try {
            GroupCollectionResponse groupResponse = graphClient.groups().get(requestConfiguration -> {
                if (!StringUtils.isEmpty(queryString)) {
                    requestConfiguration.queryParameters.filter = "startswith(displayName, '" + queryString + "')";
                }
            });
            List<Group> groups = groupResponse.getValue();
            List<MSGroup> msGroups = new ArrayList<>();
            groups.forEach(group -> {
                String name = group.getDisplayName();
                String id = group.getId();
                MSGroup msGroup = new MSGroup();
                msGroup.setId(id);
                msGroup.setDisplayName(name);
                msGroups.add(msGroup);
            });
            return msGroups;
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }

    public String listToString(List<String> stringList) {
        return String.join(",", stringList);
    }

    public List<String> stringToList(String str) {
        return Arrays.asList(str.split(","));
    }

}
