package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.mapper.VppCompanyMapper;
import cn.kdan.pdf.tech.core.model.VppCompany;
import cn.kdan.pdf.tech.core.model.VppCompanyExample;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.pojo.dto.SetGeneralDTO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMenberVO;
import cn.kdan.pdf.tech.core.service.VppCompanyService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import cn.kdan.pdf.tech.core.service.VppSetService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 设置接口
 * @author Bob
 */
@Slf4j
@Service
public class VppCompanyServiceImpl implements VppCompanyService {

    @Autowired
    private VppCompanyMapper vppCompanyMapper;
    @Autowired
    private VppMemberService vppMemberService;

    @Override
    public VppCompany getUserCompany() {
        VppMember member = vppMemberService.getCurrentUser();
        return vppCompanyMapper.selectByPrimaryKey(member.getCompanyId());
    }

    @Override
    public VppCompany getById(String id) {
        return vppCompanyMapper.selectByPrimaryKey(id);
    }

    /**
     * 获取当前登录用户的公司所有用户，除超管
     * @return
     */
    @Override
    public List<VppTeamMenberVO> getComponyMenber() {
        VppMember member = vppMemberService.getCurrentUser();
        List<VppMember> componyMenber = vppMemberService.getComponyMenber(member.getCompanyId());
        if (CollectionUtils.isEmpty(componyMenber)) {
            return Collections.emptyList();
        }
        // 过滤超管
        List<VppMember> collect = componyMenber.stream().filter(c -> !c.getRole().contains("1")).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(collect)) {
            return Collections.emptyList();
        }
        List<VppTeamMenberVO> vppTeamMenberVOList = Lists.newArrayList();
        for (VppMember vppMember : collect) {
            VppTeamMenberVO vppTeamMenberVO = new VppTeamMenberVO();
            vppTeamMenberVO.setId(vppMember.getId());
            vppTeamMenberVO.setEmail(vppMember.getEmail());
            vppTeamMenberVO.setFullName(vppMember.getFullName());
            vppTeamMenberVOList.add(vppTeamMenberVO);
        }
        return vppTeamMenberVOList;
    }

    @Override
    public List<VppCompany> getByName(String name) {
        VppCompanyExample example = new VppCompanyExample();
        example.createCriteria().andNameEqualTo(name);
        return vppCompanyMapper.selectByExample(example);
    }
}
