package cn.kdan.pdf.tech.core.service.impl;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.kdan.pdf.tech.core.constant.VppLicenseCodeConstant;
import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.mapper.ext.ExtVppDeviceMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.pojo.vo.*;
import cn.kdan.pdf.tech.core.service.*;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 *  Dashboard数据查询接口
 * @author Bob
 */
@Slf4j
@Service
public class VppDashboardServiceImpl implements VppDashboardService {

    @Autowired
    private VppTeamService vppTeamService;
    @Autowired
    private ProductService productService;
    @Autowired
    private VppLicenseCodeService vppLicenseCodeService;
    @Autowired
    private VppDeviceService vppDeviceService;
    @Autowired
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Autowired
    private VppMemberService vppMemberService;

    @Override
    public DashboardOverviewVO getOverview() {
        // 获取当前登录用户
        VppMember currentUser = vppMemberService.getCurrentUser();
        DashboardOverviewVO dashboardOverviewVO = new DashboardOverviewVO();
        List<LicenseCodes> vppLicenseCodeList = vppLicenseCodeService.getByCompanyId(currentUser.getCompanyId());
        // 已分配，包含已激活
        int assignedLicenses = (int) vppLicenseCodeList.stream().filter(c -> c.getValidFlag().equals(LicenseCodeStatusEnum.ASSIGNED.code()) || c.getValidFlag().equals(LicenseCodeStatusEnum.ACTIVE.code())).count();
        // 可分配
        int availableLicenses = (int) vppLicenseCodeList.stream().filter(c -> c.getValidFlag().equals(LicenseCodeStatusEnum.UNASSIGNED.code()) || c.getValidFlag().equals(LicenseCodeStatusEnum.CANCELED.code())).count();
        List<LicenseCodes> collect = vppLicenseCodeList.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())&& !c.getValidFlag().equals(LicenseCodeStatusEnum.EXPIRED.code())).collect(Collectors.toList());
        // 已激活
        int activatedDevices = collect.stream().mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();

        dashboardOverviewVO.setTotalLicenses(vppLicenseCodeList.size());
//        dashboardOverviewVO.setAssignedLicenses(assignedLicenses);
//        dashboardOverviewVO.setAvailableLicenses(availableLicenses);
        dashboardOverviewVO.setActivatedDevices(activatedDevices);

        // 总购买可激活设备数
        int availableDevices = collect.stream().mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
        dashboardOverviewVO.setAvailableDevices(availableDevices);
        // 剩余可以激活
        dashboardOverviewVO.setRemainingAvailableDevices(availableDevices - activatedDevices);
        return dashboardOverviewVO;
    }

    /**
     * 获取全局预览数据，区分产品
     * 区分超管和团队管理员权限
     * @param codeList 产品编码
     * @return DashboardOverviewVO
     */
    @Override
    public DashboardOverviewProductVO getOverviewProduct(List<String> codeList) {
        // 根据code筛选产品
        List<Integer> productIdList = null;
        if (CollectionUtils.isNotEmpty(codeList)) {
            List<Products> productsList = productService.getByCodeList(codeList);
            if (CollectionUtils.isNotEmpty(productsList)) {
                productIdList = productsList.stream().map(Products::getId).collect(Collectors.toList());
            }
        }
        // 获取当前登录用户
        VppMember currentUser = vppMemberService.getCurrentUser();
        DashboardOverviewProductVO dashboardOverviewProductVO = new DashboardOverviewProductVO();
        // 根据账号权限查询
        if (currentUser.getRole().contains("1")) {
            // 超管查询公司所有licence数据
            List<LicenseCodes> licenseCodesList = vppLicenseCodeService.getByCompanyId(currentUser.getCompanyId());
            if (CollectionUtils.isNotEmpty(productIdList)) {
                List<Integer> finalProductIdList = productIdList;
                List<LicenseCodes> collect = licenseCodesList.stream().filter(c -> finalProductIdList.contains(c.getProductId())).collect(Collectors.toList());
                dealWithOverviewDate(dashboardOverviewProductVO, collect);
            } else {
                dealWithOverviewDate(dashboardOverviewProductVO, licenseCodesList);
            }
        } else {
            // 管理员查询管理的团队
            List<VppRTeamMemberRoleVO> vppRTeamMemberRoleVOS = vppRTeamMemberRoleService.listByMemberIdAdmin(currentUser.getId());
            List<String> teamIdList = vppRTeamMemberRoleVOS.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(teamIdList)) {
                dashboardOverviewProductVO.setTotalLicenses(0);
                dashboardOverviewProductVO.setAvailableLicenses(0);
                dashboardOverviewProductVO.setActivatedDevices(0);
                return dashboardOverviewProductVO;
            }
            // 查询团队下的licence数据
            List<LicenseCodes> licenseCodesList = null;
            if (CollectionUtils.isNotEmpty(productIdList)) {
                licenseCodesList = vppLicenseCodeService.getByTeamIdsAndProduceIds(currentUser.getCompanyId(), teamIdList, productIdList);
            } else {
                licenseCodesList = vppLicenseCodeService.getByTeamIds(currentUser.getCompanyId(), teamIdList);
            }
            dealWithOverviewDate(dashboardOverviewProductVO, licenseCodesList);
        }
        return dashboardOverviewProductVO;
    }

    private void dealWithOverviewDate(DashboardOverviewProductVO dashboardOverviewProductVO, List<LicenseCodes> licenseCodesList) {
        dashboardOverviewProductVO.setTotalLicenses((int) licenseCodesList.stream().filter(c -> !(c.getValidFlag().equals(LicenseCodeStatusEnum.DELETED.code()))).count());
//        dashboardOverviewProductVO.setAvailableLicenses((int) licenseCodesList.stream().filter(c -> c.getValidFlag().equals(LicenseCodeStatusEnum.UNASSIGNED.code()) || c.getValidFlag().equals(LicenseCodeStatusEnum.CANCELED.code())).count());
        List<LicenseCodes> collect = licenseCodesList.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).collect(Collectors.toList());
        int activatedDevices = collect.stream().mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
        dashboardOverviewProductVO.setActivatedDevices(activatedDevices);
        // 总购买可激活设备数
        int availableDevices = collect.stream().mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
        dashboardOverviewProductVO.setRemainingAvailableDevices(availableDevices - activatedDevices);
//        // 判断序列码有效期是否超过50年
//        LicenseCodes licenseCodes = licenseCodesList.stream().max(Comparator.comparing(LicenseCodes::getEndUpAt)).orElse(null);
        // 判断序列码是否有购买，只有试用才有有效期
        List<Short> collect1 = licenseCodesList.stream().map(LicenseCodes::getType).distinct().collect(Collectors.toList());
        if (collect1.contains(Short.valueOf("1"))) {
            dashboardOverviewProductVO.setValidPeriod("--");
            dashboardOverviewProductVO.setExpireDate("--");
            dashboardOverviewProductVO.setTimeLeft("--");
            return;
        }
        // 没有购买的序列码，查询最大有效期
        LicenseCodes licenseCodes = licenseCodesList.stream().max(Comparator.comparing(LicenseCodes::getEndUpAt)).orElse(null);
        if (null == licenseCodes || null == licenseCodes.getEndUpAt()) {
            dashboardOverviewProductVO.setValidPeriod("--");
            dashboardOverviewProductVO.setExpireDate("--");
            dashboardOverviewProductVO.setTimeLeft("--");
        } else {
            if (DateUtil.date().compareTo(licenseCodes.getEndUpAt()) > 0) {
                dashboardOverviewProductVO.setTimeLeft("0");
            } else {
                dashboardOverviewProductVO.setTimeLeft(String.valueOf(DateUtil.between(licenseCodes.getEndUpAt(), DateUtil.date(), DateUnit.DAY)));
            }
            int validPeriod = (int) DateUtil.between(licenseCodes.getEndUpAt(), licenseCodes.getCreatedAt(), DateUnit.DAY);
            validPeriod = validPeriod / 30;
            dashboardOverviewProductVO.setValidPeriod(String.valueOf(validPeriod));
            dashboardOverviewProductVO.setExpireDate(licenseCodes.getEndUpAt().toString());
        }
    }

    /**
     * 获取图表数据
     * 不区分账户权限
     * @param type 1today,2last 7 days,3last 30 days,4this month
     * @return DashboardChartStatisticVO
     */
    @Override
    public List<DashboardChartStatisticVO> getChartStatistics(Integer type) {
        if (null == type || type < 1 || type > 4) {
            throw new BackendRuntimeException(VppLicenseCodeConstant.PARAMETER_ERROR);
        }
        // 获取当前登录用户
        VppMember currentUser = vppMemberService.getCurrentUser();
        DashboardOverviewVO dashboardOverviewVO = new DashboardOverviewVO();
        // 判断查询时间范围
        Date startTime = null;
        Date endTime = null;
        Date now = new Date();
        switch (type) {
            case 1:
                startTime = DateUtil.beginOfDay(now);
                endTime = DateUtil.endOfDay(now);
                break;
            case 2:
                startTime = DateUtil.beginOfDay(DateUtil.offsetDay(now, -7));
                endTime = DateUtil.endOfDay(now);
                break;
            case 3:
                startTime = DateUtil.beginOfDay(DateUtil.offsetDay(now, -30));
                endTime = DateUtil.endOfDay(now);
                break;
            case 4:
                startTime = DateUtil.beginOfMonth(now);
                endTime = DateUtil.endOfMonth(now);
                break;
        }
        // 获取公司下的所有序列码
        List<LicenseCodes> vppLicenseCodeList = vppLicenseCodeService.getByCompanyId(currentUser.getCompanyId());
        List<DashboardChartStatisticVO> dashboardChartStatisticVOList = new ArrayList<>();
        for (; startTime.compareTo(endTime) < 0; startTime = DateUtil.offsetDay(startTime, 1)) {
            DashboardChartStatisticVO dashboardChartStatisticVO = new DashboardChartStatisticVO();
            dashboardChartStatisticVO.setDate(DateUtil.format(startTime, DatePattern.NORM_DATE_PATTERN));
            // 数据日期过滤
            Date finalStartTime = startTime;
            Date finalEndTime = DateUtil.endOfDay(startTime);
            int assignedLicenses = 0;
            int activatedLicense = 0;
            if (CollectionUtils.isNotEmpty(vppLicenseCodeList)) {
                // 查询授权数据
                assignedLicenses = (int) vppLicenseCodeList.stream().filter(c -> c.getAssignedDate() != null && c.getAssignedDate().compareTo(finalStartTime) > 0
                        && c.getAssignedDate().compareTo(finalEndTime) < 0 && (c.getValidFlag() == 2 || c.getValidFlag() == 3 || c.getValidFlag() == 8)).count();
                // 查询激活数据
                activatedLicense = (int) vppLicenseCodeList.stream().filter(c -> c.getActivatedDate() != null && c.getActivatedDate().compareTo(finalStartTime) > 0
                        && c.getActivatedDate().compareTo(finalEndTime) < 0 && (c.getValidFlag() == 3 || c.getValidFlag() == 8)).count();
            }
            DashboardChartStatisticVO.Licence licence = new DashboardChartStatisticVO.Licence();
            licence.setAssignedLicenses(assignedLicenses);
            licence.setActivatedLicense(activatedLicense);
            dashboardChartStatisticVO.setLicences(licence);
            dashboardChartStatisticVOList.add(dashboardChartStatisticVO);
        }
        return dashboardChartStatisticVOList;
    }

    @Override
    public DashboardTeamDataVO getTeamData() {
        // 获取当前登录用户
        VppMember currentUser = vppMemberService.getCurrentUser();
        // 查询所有团队数量
        List<VppTeam> vppTeams = vppTeamService.listByCompany(currentUser.getCompanyId());
        int totalMember = 0;
        int assignedMember = 0;
        // 如果没有团队就不用查询后面的数据
        if (CollectionUtils.isNotEmpty(vppTeams)) {
            // 查询所有团队成员数量
            List<String> teamIdList = vppTeams.stream().map(VppTeam::getId).collect(Collectors.toList());
            totalMember = vppRTeamMemberRoleService.countMenberNumByTeamIdList(teamIdList);
            // 查询有授权的用户数
            List<LicenseCodes> licenseCodesList = vppLicenseCodeService.getByTeamIds(currentUser.getCompanyId(), teamIdList);
            List<Integer> assignedMemberStatus = Arrays.asList(LicenseCodeStatusEnum.ASSIGNED.code(), LicenseCodeStatusEnum.ACTIVE.code(), LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code());
            assignedMember = (int) licenseCodesList.stream().filter(c -> assignedMemberStatus.contains(c.getValidFlag())).map(LicenseCodes::getVppMemberId).distinct().count();
        }

        DashboardTeamDataVO dashboardTeamDataVO = new DashboardTeamDataVO();
        dashboardTeamDataVO.setTotalTeam(vppTeams.size());
        dashboardTeamDataVO.setTotalMember(totalMember);
        dashboardTeamDataVO.setAssignedMember(assignedMember);
        return dashboardTeamDataVO;
    }

    /**
     * 获取团队成员数据
     * 区分超管和团队管理员权限
     * @return DashboardTeamMemberVO
     */
    @Override
    public DashboardTeamMemberVO getTeamMemberData(List<String> codeList, String teamId) {
        // 获取当前登录用户
        VppMember currentUser = vppMemberService.getCurrentUser();
        // 根据账号权限判断是否需要加管理团队限制
        List<String> manageTeamIdList = null;
        if (!currentUser.getRole().contains("1")) {
            // 管理员查询管理的团队
            List<VppRTeamMemberRoleVO> vppRTeamMemberRoleVOS = vppRTeamMemberRoleService.listByMemberIdAdmin(currentUser.getId());
            manageTeamIdList = vppRTeamMemberRoleVOS.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
        }
        // 根据code筛选产品
        List<Integer> productIdList = null;
        if (CollectionUtils.isNotEmpty(codeList)) {
            List<Products> productsList = productService.getByCodeList(codeList);
            if (CollectionUtils.isNotEmpty(productsList)) {
                productIdList = productsList.stream().map(Products::getId).collect(Collectors.toList());
            }
        }
//        List<LicenseCodes> licenseCodesList = vppLicenseCodeService.getByProduceIds(currentUser.getCompanyId(), productIdList);
//        // 筛选序列码关联的所有团队id
//        List<String> licenseTeamIdList = licenseCodesList.stream().map(LicenseCodes::getTeamId).distinct().collect(Collectors.toList());
        // 根据条件查询团队
        List<VppTeam> vppTeamList = vppTeamService.getByTeamParam(currentUser.getCompanyId(), teamId, null, manageTeamIdList);
        Integer teamMember = 0;
        Integer assignedMember = 0;
        Integer teamAvailableLicense = 0;
        Integer remainingActivatableDevices = 0;
        if (CollectionUtils.isNotEmpty(vppTeamList)) {
            List<String> teamIdList = vppTeamList.stream().map(VppTeam::getId).collect(Collectors.toList());
            // 查询团队成员数量
            teamMember = vppRTeamMemberRoleService.countMenberNumByTeamIdList(teamIdList);
            // 查询有授权的用户数
            List<LicenseCodes> teamLicenseCodesList = vppLicenseCodeService.getByTeamIds(currentUser.getCompanyId(), teamIdList);
            // 判断是否有序列码过滤
            List<Integer> assignedMemberStatus = Arrays.asList(LicenseCodeStatusEnum.ASSIGNED.code(), LicenseCodeStatusEnum.ACTIVE.code(), LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code());
            if (CollectionUtils.isNotEmpty(productIdList)) {
                List<Integer> finalProductIdList = productIdList;
                // 统计授权人数
                assignedMember = (int) teamLicenseCodesList.stream().filter(c -> assignedMemberStatus.contains(c.getValidFlag())
                        && finalProductIdList.contains(c.getProductId())).map(LicenseCodes::getVppMemberId).distinct().count();

                // 统计剩余可激活设备数
                int times = teamLicenseCodesList.stream().filter(c -> finalProductIdList.contains(c.getProductId()))
                        .mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
                int usedTimes = teamLicenseCodesList.stream().filter(c -> finalProductIdList.contains(c.getProductId()))
                        .mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
                remainingActivatableDevices = times - usedTimes;
            } else {
                // 统计授权人数
                assignedMember = (int) teamLicenseCodesList.stream().filter(c -> assignedMemberStatus.contains(c.getValidFlag()))
                        .map(LicenseCodes::getVppMemberId).distinct().count();

                // 统计剩余可激活设备数
                int times = teamLicenseCodesList.stream().mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
                int usedTimes = teamLicenseCodesList.stream().mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
                remainingActivatableDevices = times - usedTimes;
            }
        }

        DashboardTeamMemberVO dashboardTeamMemberVO = new DashboardTeamMemberVO();
        dashboardTeamMemberVO.setTeamMember(teamMember);
        dashboardTeamMemberVO.setAssignedMember(assignedMember);
        dashboardTeamMemberVO.setTeamAvailableLicense(teamAvailableLicense);
        dashboardTeamMemberVO.setRemainingActivatableDevices(remainingActivatableDevices);
        return dashboardTeamMemberVO;
    }
}
