package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.mapper.VppMemberMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.params.VppMemberRegisterParam;
import cn.kdan.pdf.tech.core.params.VppMemberResetPwdParam;
import cn.kdan.pdf.tech.core.params.VppMemberUpdatePwdParam;
import cn.kdan.pdf.tech.core.pojo.oauth2.UserInfo;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.RSAUtils;
import cn.kdan.pdf.tech.core.utils.ThreadPoolSingleUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import constant.CommonConstant;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.http.MediaType;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;
import pojo.CustomUserDetails;
import utils.CommonUtils;
import cn.kdan.pdf.tech.core.utils.EmailUtils;
import utils.RedisUtils;

import javax.annotation.Resource;
import java.util.*;


/**
 * oauth2 用户方法类
 *
 * @author tangxiangan
 */
@Slf4j
@Service
@Transactional
public class VppMemberServiceImpl implements VppMemberService {

    @Resource
    private VppMemberMapper vppMemberMapper;
    @Resource
    private AuthService authService;
    @Resource
    private VppTeamService vppTeamService;
    @Resource
    private VppCompanyService vppCompanyService;
    @Resource
    private RedisUtils redisUtils;
    @Autowired
    private EmailUtils emailUtils;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
//    @Value("${webUrl}")
//    private String  webUrl;
//    @Value("${loginUrl}")
//    private String  loginUrl;
    // 运营后台地址，图片地址emailImagesUrl
    @Value("${readerproUrl}")
    private String readerproUrl;
    @Value("${emailImagesUrl}")
    private String emailImagesUrl;

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    public static Authentication getAuthentication() {
        return SecurityContextHolder.getContext().getAuthentication();
    }

    private static void checkPwdLength(String password) {
        if (password.length() < CommonConstant.MIN_PASSWORD_LENGTH || password.length() > CommonConstant.MAX_PASSWORD_LENGTH) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_PASSWORD_NUMBER);
        }
    }

    @Override
    public VppMember getById(String userId) {
        return vppMemberMapper.selectByPrimaryKey(userId);
    }

    @Override
    public VppMember getByAccount(String account) {
        VppMemberExample example = new VppMemberExample();
        example.createCriteria().andEmailEqualTo(account).andValidFlagEqualTo(ValidStatusEnum.VALID.value()).andIsSsoIsNull();
        List<VppMember> users = vppMemberMapper.selectByExample(example);
        return CollectionUtils.isEmpty(users) ? null : users.get(0);
    }

    @Override
    public VppMember getByAccountSSO(String account) {
        VppMemberExample example = new VppMemberExample();
        example.createCriteria().andEmailEqualTo(account).andIsSsoEqualTo(ValidStatusEnum.VALID.value());
        List<VppMember> users = vppMemberMapper.selectByExample(example);
        return CollectionUtils.isEmpty(users) ? null : users.get(0);
    }


    @Override
    public VppMember create(VppMemberRegisterParam param) {
        //用户注册后都为普通用户，没有初始团队，团队将在申请试用后创建团队
        //校验
        validRegisterParam(param);
        RSAUtils rsaUtils = new RSAUtils();
        //注册
        VppMember vppMember = new VppMember().withId(CommonUtils.generateId()).withFullName(param.getUsername())
                .withEmail(param.getEmail()).withValidFlag(ValidStatusEnum.VALID.value())
                .withCreatedAt(new Date()).withIndustry(param.getIndustry())
                .withUpdatedAt(new Date())
                .withArea(param.getArea())
                .withSubscriberType(0).withDigestPassword(rsaUtils.encrypt(param.getPassword())).withUpdatedAt(new Date());
        if(!StringUtils.isEmpty(param.getIsSuperAdmin())&&param.getIsSuperAdmin()){
            //todo 创建默认团队
            vppMember.withRole(RoleEnum.SUPER_ADMIN.code());
            vppMember.withCompanyId(param.getCompany());
            //插入与默认团队的关联关系
            VppRTeamMemberRole vppRTeamMemberRole = new VppRTeamMemberRole().withMemberId(vppMember.getId())
                    .withRoleId(RoleEnum.SUPER_ADMIN.code()).withTeamId("1");
            vppRTeamMemberRoleService.create(vppRTeamMemberRole);
        }else {
            //如果不传团队id则为普通成员注册
            if (CollectionUtils.isEmpty(param.getTeamIds())) {
                vppMember.withRole(RoleEnum.TEAM_MEMBER.code());
            } else {
                //如果传了团队id，则注册并且加入团队
                vppMember.withRole(RoleEnum.TEAM_ADMIN.code());
                //用户加入团队所在的公司
                VppTeam vppTeam = vppTeamService.getById(param.getTeamIds().get(0));
                vppMember.withCompanyId(vppTeam.getCompanyId());
                //有团队id就加入团队(插入团队关系表)
                param.getTeamIds().forEach(item->{
                    VppRTeamMemberRole vppRTeamMemberRole = new VppRTeamMemberRole().withMemberId(vppMember.getId())
                            .withRoleId(RoleEnum.TEAM_ADMIN.code()).withTeamId(item).withId(CommonUtils.generateId());
                    vppRTeamMemberRoleService.create(vppRTeamMemberRole);
                });
            }
        }
        vppMemberMapper.insert(vppMember);
        vppMember.setDigestPassword(null);
        return vppMember;
    }

    @Override
    public Boolean createTrial(VppMemberRegisterParam param) {
        // 公司名是否已存在
        if (!ObjectUtils.isEmpty(vppCompanyService.getByName(param.getCompany()))) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_COMPANY_EXIST);
        }
        // 校验
        validRegisterParam(param);
        // 发送http请求
        RestTemplate restTemplate = new RestTemplate();
        String url = emailImagesUrl + "/api/vpp_members/register";
        String jsonString = JSON.toJSONString(param);
        log.info("调用运营后台创建试用用户接口 url:{},json:{}", url, jsonString);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set(HttpHeaders.ACCEPT, MediaType.APPLICATION_JSON_VALUE);
        HttpEntity<String> request = new HttpEntity<>(jsonString, headers);
        ResponseEntity<JSONObject> response = restTemplate
                .exchange(url, HttpMethod.POST, request, JSONObject.class);
        log.info("result:" + response.getStatusCodeValue());
        if (!response.getStatusCode().is2xxSuccessful()) {
            log.error("调用运营后台创建试用用户接口失败，返回状态码：{}，返回内容：{}", response.getStatusCode(), response.getBody());
            return null;
        }
        return Boolean.TRUE;
    }

    @Override
    public void insert(VppMember vppMember) {
        vppMemberMapper.insert(vppMember);
    }

    @Override
    public void update(VppMember vppMember) {
        vppMemberMapper.updateByPrimaryKeySelective(vppMember);
    }

    @Override
    public void updateAll(VppMember vppMember) {
        vppMemberMapper.updateByPrimaryKey(vppMember);
    }



    @Override
    public boolean resetPassword(VppMemberResetPwdParam params) {
        VppMember member = getByAccount(params.getEmail());
        //判断账户是否存在
        if (ObjectUtils.isEmpty(member)) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_EMAIL_NOT_EXIST);
        }
        String newPassWord = CommonUtils.randomPassword(12);
        member.setDigestPassword(new RSAUtils().encrypt(newPassWord));
        member.setUpdatedAt(new Date());
        vppMemberMapper.updateByPrimaryKeySelective(member);
        //todo 重置密码
        ThreadPoolSingleUtil.getInstance().excute(() -> {
            String text = String.format(CaptchaActionEnum.FORGET_PASSWORD.text(), member.getFullName(), member.getEmail(),newPassWord);
            // 替换图片动态地址和取消订阅链接
            text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + params.getEmail());
            emailUtils.sendMail(CaptchaActionEnum.FORGET_PASSWORD.subject(), text, Collections.singletonList(params.getEmail()));
        });
        return false;
    }

    @Override
    public String getCompanySuperAdmin(String companyId) {
        return getSuperAdmin(companyId).getEmail();
    }

    @Override
    public VppMember getSuperAdmin(String companyId) {
        VppMemberExample example = new VppMemberExample();
        example.createCriteria().andCompanyIdEqualTo(companyId).andValidFlagEqualTo(ValidStatusEnum.VALID.value()).andRoleLike("%"+RoleEnum.SUPER_ADMIN.code()+"%");
        List<VppMember> users = vppMemberMapper.selectByExample(example);
        return users.get(0);
    }

    /**
     * 获取当前登录用户的公司所有用户
     * @param companyId 公司id
     * @return
     */
    @Override
    public List<VppMember> getComponyMenber(String companyId) {
        VppMemberExample example = new VppMemberExample();
        example.createCriteria().andCompanyIdEqualTo(companyId).andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        return vppMemberMapper.selectByExample(example);
    }

    @Override
    public boolean updatePassword(VppMemberUpdatePwdParam param) {
        if (!param.getNewPassword().equals(param.getNewPasswordConfirm())) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_PASSWORD_ERROR);
        }
        VppMember member = getCurrentUser();
        RSAUtils rsaUtils = new RSAUtils();
        if (!rsaUtils.decrypt(member.getDigestPassword()).equals(param.getPassword())) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_PASSWORD_MATCH_ERROR);
        }
        checkPwdLength(param.getNewPassword());
        member.setDigestPassword( new RSAUtils().encrypt(param.getNewPassword()));
        member.setUpdatedAt(new Date());
        vppMemberMapper.updateByPrimaryKeySelective(member);
        return Boolean.TRUE;
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @Override
    public VppMember getCurrentUser() {
        Authentication authentication = getAuthentication();
        Object principal = authentication.getPrincipal();
        try {
            String userId = ((CustomUserDetails) (principal)).getId();
            return vppMemberMapper.selectByPrimaryKey(userId);
        } catch (Exception e) {
            log.info("getCurrentUser获取用户失败：{}", e.getMessage());
            return null;
        }
    }

    @Override
    public Boolean checkExist(String account) {
        VppMember member = getByAccount(account);
        if (ObjectUtils.isEmpty(member)) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public void updateMemberRole(String memberId, String role) {
        VppMember vppMember = getById(memberId);
        if (!vppMember.getRole().contains(role)) {
            if (ObjectUtils.isEmpty(vppMember.getRole())) {
                vppMember.setRole(role);
            } else {
                vppMember.setRole(vppMember.getRole() + "," + role);
            }
        }
    }

    @Override
    public void loginOut() {

    }

    @Override
    public void deleteDirectoryMember(String directoryId) {
        VppMemberExample example = new VppMemberExample();
        example.createCriteria().andDirectoryIdEqualTo(directoryId);
        vppMemberMapper.deleteByExample(example);
    }
    @Override
    public void delete(String memberId) {
        vppMemberMapper.deleteByPrimaryKey(memberId);
    }

    private void validRegisterParam(VppMemberRegisterParam param) {
        //邮箱是否被注册 前后两次密码 密码位数 验证邮箱格式 邮件验证码错误
        if (!ObjectUtils.isEmpty(getByAccount(param.getEmail()))) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_EMAIL_EXIST);
        }
        //验证邮箱格式
        String emailRegex = CommonConstant.emailRegex;
        if (!param.getEmail().matches(emailRegex)) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_EMAIL_ERROR);
        }
        //前后两次密码
        if (!param.getPassword().equals(param.getConfirmPassword())) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_PASSWORD_ERROR);
        }
        //密码位数
        checkPwdLength(param.getPassword());
        //邮件验证码错误
        if (!authService.checkVerifyCode(CaptchaActionEnum.USER_REGISTER.value(), param.getEmail(), param.getVerifyCode())) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_FALSE_CODE);
        } else {
            // 验证成功删除验证码缓存
            redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.USER_REGISTER.value(), param.getEmail());
        }
        if(!StringUtils.isEmpty(param.getInviteEmailCode())&&!authService.checkVerifyCode(CaptchaActionEnum.USER_REGISTER_FOR_ADD_ADMIN.value(), param.getEmail(), param.getInviteEmailCode())) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_FALSE_EMAIL_CODE);
        }else{
            // 验证成功删除验证码缓存
            redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.USER_REGISTER_FOR_ADD_ADMIN.value(), param.getEmail());
        }
    }

    @Override
    public UserInfo convert(VppMember members) {
        UserInfo userInfo = null;
        if (members != null) {
            userInfo = new UserInfo();
            BeanUtils.copyProperties(members, userInfo);
        }

        return userInfo;
    }


    @Override
    public UserInfo detail(VppMember user) {
        return this.convert(user);
    }

}
