package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.VppDeviceConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.tech.core.enums.ProductNameEnum;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.mapper.ext.ExtVppRTeamMemberRoleMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.pojo.dto.AddTeamMemberDTO;
import cn.kdan.pdf.tech.core.pojo.vo.VppRTeamMemberRoleVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamAdminVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMemberVO;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.EmailUtils;
import cn.kdan.pdf.tech.core.utils.ExcelUtils;
import cn.kdan.pdf.tech.core.utils.FileUtil;
import cn.kdan.pdf.tech.core.utils.ThreadPoolSingleUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;
import utils.CommonUtils;
import utils.RedisUtils;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 用户与团队的关系类
 *
 * @author tangxiangan
 */
@Slf4j
@Service
@Transactional
public class VppRTeamMemberRoleServiceImpl implements VppRTeamMemberRoleService {

    @Autowired
    private ExtVppRTeamMemberRoleMapper vppRTeamMemberRoleMapper;
    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppLicenseCodeService vppLicenseCodeService;
    @Resource
    private VppTeamService vppTeamService;
    @Resource
    private VppCompanyService vppCompanyService;
    @Autowired
    private ExtVppRTeamMemberRoleMapper extVppRTeamMemberRoleMapper;

    @Autowired
    private EmailUtils emailUtils;
    @Autowired
    private RedisUtils<String, String> redisUtils;
    @Autowired
    private AuthService authService;

    // 运营后台地址，图片地址emailImagesUrl
    @Value("${readerproUrl}")
    private String readerproUrl;
    @Value("${emailImagesUrl}")
    private String emailImagesUrl;
    @Value("${confirmAddUrl}")
    private String confirmAddUrl;

    @Override
    public void create(VppRTeamMemberRole vppRTeamMember) {
        vppRTeamMemberRoleMapper.insert(vppRTeamMember);
    }

    @Override
    public List<VppRTeamMemberRoleVO> listByMemberIdAdmin(String memberId) {
        return vppRTeamMemberRoleMapper.listMemberAdmin(memberId);
    }

    @Override
    public List<String> getTeamIdByAdminName(String admin) {
        return vppRTeamMemberRoleMapper.getTeamIdByAdminName(admin);
    }

    /**
     * 获取团队所有关联关系
     * @param teamId 用户id
     * @return List
     */
    @Override
    public List<VppRTeamMemberRole> getAllByTeamId(String teamId) {
        return vppRTeamMemberRoleMapper.getAllByTeamId(teamId);
    }

    @Override
    public List<VppRTeamMemberRole> listByMemberIdAndTeamId(String memberId, String teamId) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        example.createCriteria().andMemberIdEqualTo(memberId).andTeamIdEqualTo(teamId);
        return vppRTeamMemberRoleMapper.selectByExample(example);
    }

    @Override
    public void addTeamMember(AddTeamMemberDTO dto) {
        // 获取当前登录用户
        VppMember user = vppMemberService.getCurrentUser();
        VppMember byAccount = vppMemberService.getByAccount(dto.getMemberEmail());
//        VppTeam vppTeam = vppTeamService.getById(dto.getTeamId());
        VppCompany vppCompany = vppCompanyService.getById(user.getCompanyId());
        // 如果成员存在，校验数据
        if (null != byAccount) {
            // 判断是否存在于其他公司
            if (StringUtils.isNotEmpty(byAccount.getCompanyId())&&!byAccount.getCompanyId().equals(user.getCompanyId())) {
                throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_JOINED);
            }
            // 查询用户是否已存在于团队中
            VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
            example.createCriteria().andMemberIdEqualTo(byAccount.getId()).andRoleIdLike("3");
            List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(vppRTeamMemberRoles)) {
                throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_JOINED);
            }
            // 查询是否为本团队管理员或超管
            VppRTeamMemberRoleExample example1 = new VppRTeamMemberRoleExample();
            example1.createCriteria().andMemberIdEqualTo(byAccount.getId()).andTeamIdEqualTo(dto.getTeamId()).andRoleIdIn(Arrays.asList("1", "2"));
            List<VppRTeamMemberRole> vppRTeamMemberRoles1 = vppRTeamMemberRoleMapper.selectByExample(example1);
            if (!CollectionUtils.isEmpty(vppRTeamMemberRoles1)) {
                throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_ADMIN);
            }
        }

        // 发送邮件
        ThreadPoolSingleUtil.getInstance().excute(() -> {
            CaptchaActionEnum actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_MEMBER;
            String code = CommonUtils.generateVerifyCode();
            String url = readerproUrl + confirmAddUrl + "?teamId=" + dto.getTeamId() + "&code=" + code + "&memberEmail=" + dto.getMemberEmail() + "&memberName=" + dto.getMemberName() + "&action=" + actionEnum.value();
            if (StringUtils.isEmpty(dto.getMemberName())) {
                dto.setMemberName(dto.getMemberEmail());
            }
            String text = String.format(actionEnum.text(), dto.getMemberName(), vppCompany.getName(), vppCompany.getName(), url);
            // 替换图片动态地址和取消订阅链接
            text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + dto.getMemberEmail());
            String subject = String.format(actionEnum.subject(), user.getFullName());
            if(emailUtils.sendMail(subject, text, Collections.singletonList(dto.getMemberEmail()))) {
                redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + actionEnum.value(), dto.getMemberEmail(), code, AuthConstant.INVITE_VERIFY_CODE_KEY_EXPIRE_TIME);
            }
        });
    }

    /**
     * 导入批量添加团队成员
     * @param file 文件
     * @return
     */
    // todo 后续日志根据产品规划讲错误信息返回
    @Override
    public String addTeamMemberBatch(MultipartFile file) {
        // 判断文件类型是否正确
        if (!FileUtil.isExcel(file)) {
            throw new BackendRuntimeException(VppMemberConstant.FILE_ERROR);
        }
        //判断Excel版本
        boolean isOldExcel = FileUtil.isExcelOld(file);
        int count = 0;
        try {
            // 读取数据
            InputStream in = file.getInputStream();
            List<List<String>> dataList = ExcelUtils.read(in, isOldExcel);
            if (CollectionUtils.isEmpty(dataList) || dataList.size() == 1) {
                return "Success:0";
            }
            if (dataList.get(0).size() != 3) {
                log.error("读取批量导入团队成员数据错误，读取到的列数：{}", dataList.get(0).size());
                throw new BackendRuntimeException(VppMemberConstant.FILE_ERROR);
            }
            log.info("导入数据：{}",dataList.toString());
            // 获取当前登录用户
            VppMember user = vppMemberService.getCurrentUser();
            VppCompany vppCompany = vppCompanyService.getById(user.getCompanyId());
            // 从第二行开始读取
            for (int i = 1; i < dataList.size(); i++) {
                List<String> rowDate = dataList.get(i);
                if (CollectionUtils.isEmpty(rowDate)) {
                    log.info("第{}行数据为空", i);
                    continue;
                }
                String memberEmail = rowDate.get(0);
                if (StringUtils.isEmpty(memberEmail)) {
                    log.info("第{}行memberEmail为空", i);
                    continue;
                }
                String memberName = rowDate.get(1);
//                if (StringUtils.isEmpty(memberName)) {
//                    log.info("第{}行memberName为空", i);
//                    continue;
//                }
                String teamName = rowDate.get(2);
                if (StringUtils.isEmpty(teamName)) {
                    log.info("第{}行teamName为空", i);
                    continue;
                }
                // 查询公司内是否有这个团队
                VppTeam vppTeam = vppTeamService.getByCompanyAndName(user.getCompanyId(), teamName);
                if (null == vppTeam) {
                    log.info("第{}行团队{}不存在", i, teamName);
                    continue;
                }
                // 查询当前登录用户是否有此团队的操作权限
                if (!user.getRole().contains("1")) {
                    // 团队管理员查找管理的所有团队
                    List<VppRTeamMemberRole> vppRTeamMemberRoles = listByMemberId(user.getId());
                    boolean flag = vppRTeamMemberRoles.stream().filter(c -> c.getRoleId().equals("2")).map(VppRTeamMemberRole::getTeamId).collect(Collectors.toList()).contains(vppTeam.getId());
                    if (!flag) {
                        log.info("当前登录用户对第{}行团队{}操作权限不足", i, vppTeam.getName());
                        continue;
                    }
                }
                // 查询用户是否已存在
                VppMember vppMember = vppMemberService.getByAccount(memberEmail);
                // 如果成员存在，校验数据
                if (null != vppMember) {
                    // 判断是否存在于其他公司
                    if (StringUtils.isNotEmpty(vppMember.getCompanyId())&&!vppMember.getCompanyId().equals(user.getCompanyId())) {
                        log.info("第{}行成员{}已存在于其他公司", i, memberEmail);
                        continue;
//                        throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_JOINED);
                    }
                    // 查询用户是否已存在团队中
                    VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
                    example.createCriteria().andMemberIdEqualTo(vppMember.getId()).andRoleIdLike("3");
                    List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleMapper.selectByExample(example);
                    if (!CollectionUtils.isEmpty(vppRTeamMemberRoles)) {
                        log.info("第{}行成员{}已存在于其他团队", i, memberEmail);
                        continue;
//                        throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_JOINED);
                    }
                    // 查询是否为本团队管理员或超管
                    VppRTeamMemberRoleExample example1 = new VppRTeamMemberRoleExample();
                    example1.createCriteria().andMemberIdEqualTo(vppMember.getId()).andTeamIdEqualTo(vppTeam.getId()).andRoleIdIn(Arrays.asList("1", "2"));
                    List<VppRTeamMemberRole> vppRTeamMemberRoles1 = vppRTeamMemberRoleMapper.selectByExample(example1);
                    if (!CollectionUtils.isEmpty(vppRTeamMemberRoles1)) {
                        log.info("第{}行成员{}为本团队管理员或超管", i, memberEmail);
                        continue;
//                        throw new BackendRuntimeException(VppMemberConstant.MEMBEU_HAS_ADMIN);
                    }
                }
                count++;
                // 发送邮件
                ThreadPoolSingleUtil.getInstance().excute(() -> {
                    CaptchaActionEnum actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_MEMBER;
                    String code = CommonUtils.generateVerifyCode();
                    String url = readerproUrl + confirmAddUrl + "?teamId=" + vppTeam.getId() + "&code=" + code + "&memberEmail=" + memberEmail + "&memberName=" + memberName + "&action=" + actionEnum.value();
                    String memberEmailName = StringUtils.isEmpty(memberName) ? memberName : memberEmail;
                    String text = String.format(actionEnum.text(), memberEmailName, vppCompany.getName(), vppCompany.getName(), url);
                    // 替换图片动态地址和取消订阅链接
                    text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + memberEmail);
                    String subject = String.format(actionEnum.subject(), user.getFullName());
                    if(emailUtils.sendMail(subject, text, Collections.singletonList(memberEmail))){
                        redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + actionEnum.value(), memberEmail, code, AuthConstant.INVITE_VERIFY_CODE_KEY_EXPIRE_TIME);
                    }
                });
            }
        } catch (IOException e) {
            log.error("批量导入团队成员读取数据失败，{}", e.getMessage());
        }
        return "Success:" + count;
    }

    @Override
    public String confirmTeamMember(String memberEmail, String memberName, String teamId, String action, String code) {
        if (StringUtils.isEmpty(memberName)) {
            memberName = StringUtils.EMPTY;
        }
        // 验证code有效性
        CaptchaActionEnum captchaActionEnum = CaptchaActionEnum.getEnumByValue(action);
        authService.checkVerifyCodeByAction(captchaActionEnum, memberEmail, code);
        VppMember byAccount = vppMemberService.getByAccount(memberEmail);
        VppTeam vppTeam = vppTeamService.getById(teamId);
        // 如果成员不存在
        if (null == byAccount) {
            // 新增成员
            byAccount = new VppMember().withId(CommonUtils.generateId()).withEmail(memberEmail)
                    .withFullName(memberName).withCreatedAt(new Date()).withValidFlag("1").withRole("3")
                    .withCompanyId(vppTeam.getCompanyId());
            vppMemberService.insert(byAccount);
        } else {
            byAccount.setFullName(memberName);
            byAccount.setCompanyId(vppTeam.getCompanyId());
            vppMemberService.update(byAccount);
        }
        // 新增团队成员关联
        VppRTeamMemberRole vppRTeamMemberRole = new VppRTeamMemberRole().withId(CommonUtils.generateId()).withRoleId("3")
                .withMemberId(byAccount.getId()).withTeamId(teamId);
        vppRTeamMemberRoleMapper.insert(vppRTeamMemberRole);
        return vppTeam.getName();
    }

    @Override
    public List<VppRTeamMemberRole> listByMemberId(String memberId) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        return vppRTeamMemberRoleMapper.selectByExample(example);
    }

    @Override
    public Integer countMenberNumByTeamIdList(List<String> teamIdList) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        example.createCriteria().andTeamIdIn(teamIdList);
        List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleMapper.selectByExample(example);
        return (int) vppRTeamMemberRoles.stream().map(VppRTeamMemberRole::getMemberId).distinct().count();
    }

    @Override
    public void editTeamMember(String memberId, String teamId, String name) {
        List<VppRTeamMemberRole> vppRTeamMemberRoles = getVppRTeamMemberRoles(memberId);
        //用户只会存在一个团队中，所以只有一条数据
        VppRTeamMemberRole vppRTeamMemberRole = vppRTeamMemberRoles.get(0);
        //如果用户修改了团队才解绑团队
        if(!vppRTeamMemberRole.getTeamId().equals(teamId)){
            vppRTeamMemberRole.setTeamId(teamId);
            //更新用户团队关联表
            vppRTeamMemberRoleMapper.updateByPrimaryKeySelective(vppRTeamMemberRole);
            //如果用户有序列码，则要释放序列码解绑设备
            vppLicenseCodeService.unBindByMemberId(memberId, null);
        }
        //更新用户名
        VppMember vppMember = new VppMember().withId(memberId).withFullName(name);
        vppMemberService.update(vppMember);
    }

    private List<VppRTeamMemberRole> getVppRTeamMemberRoles(String memberId) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        example.createCriteria().andMemberIdEqualTo(memberId).andRoleIdEqualTo(RoleEnum.TEAM_MEMBER.code());
        List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleMapper.selectByExample(example);
        return vppRTeamMemberRoles;
    }

    @Transactional
    @Override
    public void deleteMember(String memberId) {
        List<VppRTeamMemberRole> vppRTeamMemberRoles = getVppRTeamMemberRoles(memberId);
        List<String> ids = new ArrayList<>();
        ids.add(vppRTeamMemberRoles.get(0).getTeamId());
        //如果删除用户，则是退出所有团队
        quitTeam(memberId, ids);
        //如果用户还有管理的团队就不退出公司
        VppMember member = vppMemberService.getById(memberId);
        //单点登录用户直接删除
        if(StringUtils.isNotEmpty(member.getIsSso())&&member.getIsSso().equals(ValidStatusEnum.VALID.value())){
            vppMemberService.delete(memberId);
            return;
        }
        List<VppRTeamMemberRoleVO> adminTeamList = listByMemberIdAdmin(memberId);
        if(CollectionUtils.isEmpty(adminTeamList)){
            //退出公司
            member.setId(memberId);
            member.setCompanyId("");
            member.setRole(RoleEnum.TEAM_MEMBER.code());
            member.setLoginTime(null);
            vppMemberService.update(member);
        }
    }

    @Override
    public void deleteAdmin(String memberId) {
      List<VppRTeamMemberRoleVO> list = listByMemberIdAdmin(memberId);
      List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
      //如果是管理员退出，也是退出所有管理的团队，但是会加入默认团队
        quitTeam(memberId, idList);
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        VppRTeamMemberRoleExample.Criteria criteria = example.createCriteria().andMemberIdEqualTo(memberId).andRoleIdEqualTo(RoleEnum.TEAM_MEMBER.code());
        List<VppRTeamMemberRole> roles = vppRTeamMemberRoleMapper.selectByExample(example);
        //如果用户不是普通团队成员,则加入默认团队
        if(CollectionUtils.isEmpty(roles)){
            VppRTeamMemberRole role = new VppRTeamMemberRole();
            role.setId(CommonUtils.generateId());
            role.setMemberId(memberId);
            VppTeam defaultTeam = vppTeamService.getDefaultTeam(memberId);
            role.setTeamId(defaultTeam.getId());
            role.setRoleId(RoleEnum.TEAM_MEMBER.code());
            vppRTeamMemberRoleMapper.insert(role);
        }
        VppMember member = new VppMember();
        member.setId(memberId);
        member.setRole(RoleEnum.TEAM_MEMBER.code());
        member.setLoginTime(null);
        vppMemberService.update(member);
    }

    @Resource
    @Lazy
    private TokenStore tokenStore;

    @Override
    public void logOutUserNoAdmin(String memberId) {
        getOnlineUserList();
//        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
//        if (oAuth2AccessToken != null) {
//            OAuth2RefreshToken oAuth2RefreshToken = oAuth2AccessToken.getRefreshToken();
//            //从tokenStore中移除token
//            tokenStore.removeAccessToken(oAuth2AccessToken);
//            tokenStore.removeRefreshToken(oAuth2RefreshToken);
//            tokenStore.removeAccessTokenUsingRefreshToken(oAuth2RefreshToken);
//        }
    }

    public static final String REDIS_AUTH_KEY="auth:";

    public void getOnlineUserList() {
        //oauth存放的所有keys
        // Set<String> keys = redisTemplate.keys(OnlineConstants.REDIS_AUTH_KEY + "*");
        //经elliotxiang提醒，keys会影响redis使用，修改为scan
        Set<String> keys = scan(REDIS_AUTH_KEY);
        for (String key : keys) {
            String token = key.substring(key.indexOf(":") + 1);
            OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
        }
//        for (String key : keys) {
//            //keys = "auth:" + token
//            //根据token获取OAuth2AccessToken
//            String token = key.substring(key.indexOf(":") + 1);
//            //根据token值获取OAuth2AccessToken对象
//            OAuth2AccessToken oAuth2AccessToken = redisTokenStore.readAccessToken(token);
//
//            //获取用户id，本项目用户id存放在OAuth2AccessToken对象中的additionalInformation属性里。
//            //根据实际情况修改
//            Map<String, Object> map = oAuth2AccessToken.getAdditionalInformation();
//            if (map != null) {
//                //获取用户id
//                Long userId = (Long) map.get("openid");
//                if (userId != null) {
//                    //获取用户信息
//                    BaseUser user = this.userService.getUserById(userId);
//                    //获取用户登录日志
//                    BaseAccountLogs logs = this.accountLogsService.getUserNewestLogin(userId);
//
//                    list.add(new OnlineUserVo(token, userId, user.getFullName(),
//                            logs.getLoginTime(),
//                            oAuth2AccessToken.getExpiration(),
//                            oAuth2AccessToken.getExpiresIn()));
//                }
//            }
//        }
    }

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    public Set<String> scan(String matchKey) {
        Set<String> keys = (Set<String>) redisTemplate.execute((RedisCallback<Set<String>>) connection -> {
            Set<String> keysTmp = new HashSet<>();
            Cursor<byte[]> cursor = connection.scan(new ScanOptions.ScanOptionsBuilder().match(matchKey + "*").count(1000).build());
            while (cursor.hasNext()) {
                keysTmp.add(new String(cursor.next()));
            }
            return keysTmp;
        });

        return keys;
    }
    @Override
    public void cleanTeamAdmin(String teamId) {
        extVppRTeamMemberRoleMapper.cleanTeamAdmin(teamId);
    }

    @Override
    public void cleanTeamMember(String teamId) {
        extVppRTeamMemberRoleMapper.cleanTeamMember(teamId);
    }

    @Override
    public void changeTeamMember(String teamId, String toTeamId) {
        extVppRTeamMemberRoleMapper.changeTeamMember(teamId, toTeamId);
    }

    @Override
    public int deleteByExample(VppRTeamMemberRoleExample example) {
        return vppRTeamMemberRoleMapper.deleteByExample(example);
    }

    @Override
    public void quitTeam(String memberId, List<String> teamIds) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        VppRTeamMemberRoleExample.Criteria criteria = example.createCriteria().andMemberIdEqualTo(memberId);
        if (!CollectionUtils.isEmpty(teamIds)) {
            criteria.andTeamIdIn(teamIds);
        }
        //更新用户团队关联表
        vppRTeamMemberRoleMapper.deleteByExample(example);
        //如果用户有序列码，则要释放序列码解绑设备
        vppLicenseCodeService.unBindByMemberId(memberId, teamIds);
    }

    @Override
    public void update(VppRTeamMemberRole vppRTeamMemberRole) {
        vppRTeamMemberRoleMapper.updateByPrimaryKeySelective(vppRTeamMemberRole);
    }

    @Override
    public PageInfo<VppTeamAdminVO> pageForAdmin(Integer page, Integer pageSize, String teamId, String role, String queryString) {
        VppMember member = vppMemberService.getCurrentUser();
        List<String> roleIds = new ArrayList<>();
        if(ObjectUtils.isEmpty(role)){
            roleIds.add(RoleEnum.SUPER_ADMIN.code());
            roleIds.add(RoleEnum.TEAM_ADMIN.code());
        }else{
            roleIds.add(role);
        }
        List<String> teamIds = new ArrayList<>();
        if(ObjectUtils.isEmpty(teamId)){
            List<VppRTeamMemberRoleVO> list = listByMemberIdAdmin(member.getId());
            List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            teamIds = idList;
        }else{
            teamIds.add(teamId);
        }
        if(CollectionUtils.isEmpty(teamIds)){
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_NO_TEAM);
        }
        PageHelper.startPage(page, pageSize);
        return new PageInfo<>(vppRTeamMemberRoleMapper.memberList( teamIds, queryString, roleIds));
    }


    @Override
    public PageInfo<VppTeamMemberVO> pageForMember(Integer page, Integer pageSize, String teamId, String status, String productCode, String queryString, String isSso, String domain) {
        List<String> teamIds = new ArrayList<>();
        VppMember member = vppMemberService.getCurrentUser();
        if(ObjectUtils.isEmpty(teamId)){
            List<VppRTeamMemberRoleVO> list = listByMemberIdAdmin(member.getId());
            List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            teamIds = idList;
        }else{
            teamIds.add(teamId);
        }
        if(CollectionUtils.isEmpty(teamIds)){
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_NO_TEAM);
        }
        PageHelper.startPage(page, pageSize);
        return new PageInfo<>(vppRTeamMemberRoleMapper.listForMember(teamIds, null, queryString, status, productCode, isSso,domain ));
    }

    @Override
    public List<VppTeamMemberVO> listForMember(String teamId, String status, String productCode, String queryString, String isSso, String domain) {
        List<String> teamIds = new ArrayList<>();
        if(ObjectUtils.isEmpty(teamId)){
            VppMember member = vppMemberService.getCurrentUser();
            List<VppRTeamMemberRoleVO> list = listByMemberIdAdmin(member.getId());
            List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            teamIds = idList;
        }else{
            teamIds.add(teamId);
        }
        if(CollectionUtils.isEmpty(teamIds)){
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_NO_TEAM);
        }
        List<VppTeamMemberVO> vppTeamMemberVOS = vppRTeamMemberRoleMapper.listForMember(teamIds, null, queryString, status, productCode, isSso, domain);

        for (VppTeamMemberVO vppTeamMemberVO : vppTeamMemberVOS) {
            if (StringUtils.isEmpty(vppTeamMemberVO.getProductNames())) {
                continue;
            }
            String[] split = vppTeamMemberVO.getProductNames().split(",");
            if (ObjectUtils.isEmpty(split)) {
                continue;
            }
            String[] productNamesArray = new String[split.length];
            for (int i = 0; i < split.length; i++) {
                productNamesArray[i] = ProductNameEnum.getEnumByDatabaseName(split[i]).viewName();
            }
            vppTeamMemberVO.setProductNames(String.join(",", productNamesArray));
        }
        return vppTeamMemberVOS;
    }

    @Override
    public String getMemberTeam(String memberId) {
        VppMember member = vppMemberService.getById(memberId);
        List<VppRTeamMemberRole> list = getVppRTeamMemberRoles(member.getId());
        return list.size()>0 ?list.get(0).getTeamId() : "";
    }

    @Override
    public List<VppTeamAdminVO> listByTeamId(String teamId) {
        List<String> list =new ArrayList<>();
        list.add(teamId);
        List<VppTeamAdminVO> vppTeamAdminVOS = vppRTeamMemberRoleMapper.memberList(list, null, null);
        if (CollectionUtils.isEmpty(vppTeamAdminVOS)) {
            return Collections.emptyList();
        } else {
            TreeSet<VppTeamAdminVO> collect = vppTeamAdminVOS.stream().collect(Collectors.toCollection(
                    () -> new TreeSet<>(
                            Comparator.comparing(tc -> tc.getEmail())
                    )
            ));
            return new ArrayList<>(collect);
        }
    }

    @Override
    public List<VppRTeamMemberRole> listByTeamIdWithoutSuperAdmin(String teamId) {
        VppRTeamMemberRoleExample example = new VppRTeamMemberRoleExample();
        example.createCriteria().andTeamIdEqualTo(teamId).andRoleIdNotEqualTo(RoleEnum.SUPER_ADMIN.code());
        return vppRTeamMemberRoleMapper.selectByExample(example);
    }


    @Override
    public void removeSSOMembers(List<String> memberIds) {
        memberIds.forEach(memberId->{
            List<VppRTeamMemberRole> vppRTeamMemberRoles = getVppRTeamMemberRoles(memberId);
            List<String> ids = new ArrayList<>();
            ids.add(vppRTeamMemberRoles.get(0).getTeamId());
            //sso用户只有一个默认团队，只需要解绑设备序列码，并且修改用户数据
            vppLicenseCodeService.unBindByMemberId(memberId, ids);
            VppMember member = vppMemberService.getById(memberId);
            member.setRole(RoleEnum.TEAM_MEMBER.code());
            member.setValidFlag(ValidStatusEnum.INVALID.value());
            member.setLoginTime(null);
            vppMemberService.updateAll(member);
        });
    }

    @Override
    public void revokeSSOMember(String memberId) {
        VppMember member = vppMemberService.getById(memberId);
        member.setRole(RoleEnum.TEAM_MEMBER.code());
        member.setValidFlag(ValidStatusEnum.REVOKED.value());
        member.setLoginTime(null);
        vppMemberService.updateAll(member);
    }

    @Override
    public void reductionSSOMember(List<String> memberIds) {
        memberIds.forEach(memberId->{
            VppMember member = new VppMember();
            member.setId(memberId);
            member.setRole(RoleEnum.TEAM_MEMBER.code());
            member.setValidFlag(ValidStatusEnum.VALID.value());
            vppMemberService.update(member);
        });
    }

    @Override
    public void accessSSOMember(String memberId) {
        VppMember member = new VppMember();
        member.setId(memberId);
        member.setRole(RoleEnum.TEAM_MEMBER.code());
        member.setValidFlag(ValidStatusEnum.VALID.value());
        vppMemberService.update(member);
    }


}
