package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.constant.VppTeamConstant;
import cn.kdan.pdf.tech.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.mapper.ext.ExtVppTeamMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.pojo.dto.CreateTeamDTO;
import cn.kdan.pdf.tech.core.pojo.dto.EditTeamDTO;
import cn.kdan.pdf.tech.core.pojo.vo.*;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.ThreadPoolSingleUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import cn.kdan.pdf.tech.core.utils.RSAUtils;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import constant.CommonConstant;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import utils.CommonUtils;
import cn.kdan.pdf.tech.core.utils.EmailUtils;
import utils.RedisUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * oauth2 用户方法类
 *
 * @author tangxiangan
 */
@Slf4j
@Service
@Transactional
public class VppTeamServiceImpl implements VppTeamService {

    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppCompanyService vppCompanyService;
    @Autowired
    private EmailUtils emailUtils;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Resource
    private VppLicenseCodeService vppLicenseCodeService;
    @Autowired
    private ExtVppTeamMapper vppTeamMapper;
    @Autowired
    private VppDeviceService vppDeviceService;
    @Resource
    private AuthService authService;
    @Resource
    private VppCompanyService companyService;
    @Value("${webUrl}")
    private String  webUrl;
    @Value("${createUrl}")
    private String  createUrl;
    @Value("${loginUrl}")
    private String  loginUrl;
    // 运营后台地址，图片地址emailImagesUrl
    @Value("${readerproUrl}")
    private String readerproUrl;
    @Value("${emailImagesUrl}")
    private String emailImagesUrl;
    @Autowired
    private RedisUtils<String, String> redisUtils;
    private static final ThreadPoolExecutor SEND_MAIL_THREAD_POOL = new ThreadPoolExecutor( Runtime.getRuntime().availableProcessors(),
            Runtime.getRuntime().availableProcessors(),
            1L,
            TimeUnit.MINUTES,
            new LinkedBlockingQueue<>(300), new ThreadFactoryBuilder()
            .setNameFormat("TEAM_SEND_MAIL_THREAD_POOL").build(), new ThreadPoolExecutor.AbortPolicy());

    /**
     * 团队管理列表-分页
     * @param teamId 团队id
     * @param keyword 模糊查询关键字（团队描述、团队管理员）
     * @return 团队数据
     */
    @Override
    public PageInfo<VppTeamVO> getManageTeamList(Integer page, Integer pageSize, String teamId, String keyword) {
        List<String> manageAdminTeamIdList = null;
        if (StringUtils.isNotEmpty(keyword)) {
            keyword = keyword.concat("%");
            manageAdminTeamIdList = vppRTeamMemberRoleService.getTeamIdByAdminName(keyword);
        }
        VppMember member = vppMemberService.getCurrentUser();
        // 判断当前登录用户是否超管,过滤团队
        List<String> teamIdList = new ArrayList<>();
        if (!member.getRole().contains("1")) {
            teamIdList.add("0.0");
            // 团队管理员查找管理的所有团队
            List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(member.getId());
            teamIdList.addAll(vppRTeamMemberRoles.stream().filter(c -> c.getRoleId().equals("2")).map(VppRTeamMemberRole::getTeamId).collect(Collectors.toList()));
        }
        PageHelper.startPage(page, pageSize);
        PageInfo<VppTeamVO> vppTeamVOPageInfo = new PageInfo<>(vppTeamMapper.getManageTeamList(member.getCompanyId(), teamId, keyword, teamIdList, manageAdminTeamIdList));
        // 查询团队成员数量和管理员信息，统计团队序列码数量
        for (VppTeamVO vppTeamVO : vppTeamVOPageInfo.getList()) {
            Integer memberAmount = 0;
            List<String> teamAdmin = new ArrayList<>();
            List<VppRTeamMemberRole> allByTeamId = vppRTeamMemberRoleService.getAllByTeamId(vppTeamVO.getId());
            if (!CollectionUtils.isEmpty(allByTeamId)) {
                memberAmount = allByTeamId.size();
                List<VppRTeamMemberRole> collect = allByTeamId.stream()
                        .filter(c -> (c.getRoleId().equals("1") || c.getRoleId().equals("2"))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(collect)) {
                    for (VppRTeamMemberRole vppRTeamMemberRole : collect) {
                        VppMember vppMember = vppMemberService.getById(vppRTeamMemberRole.getMemberId());
                        teamAdmin.add(vppMember.getEmail());
                    }
                }
            }
            vppTeamVO.setTeamAdmin(teamAdmin);
            vppTeamVO.setMemberAmount(memberAmount);
            vppTeamVO.setCreateTime(vppTeamVO.getCreateTime().substring(0, 19));

            // 统计团队序列码数量
            List<LicenseCodes> byTeamIds = vppLicenseCodeService.getByTeamIds(member.getCompanyId(), Collections.singletonList(vppTeamVO.getId()));
            List<LicenseCodes> collect = byTeamIds.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(byTeamIds) && CollectionUtils.isEmpty(collect)) {
                vppTeamVO.setLicenseAmount(0);
                vppTeamVO.setAvailableDevices(0);
                vppTeamVO.setActivatedDevices(0);
            } else {
                int total = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
                int used = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
                vppTeamVO.setLicenseAmount(byTeamIds.size());
                vppTeamVO.setAvailableDevices(total - used);
                vppTeamVO.setActivatedDevices(used);
            }

            // 设置账号操作权限
            if (member.getRole().contains("1")) {
                vppTeamVO.setIsChange(1);
            } else {
                if (teamIdList.contains(vppTeamVO.getId())) {
                    vppTeamVO.setIsChange(1);
                } else {
                    vppTeamVO.setIsChange(0);
                }
            }
        }
        return vppTeamVOPageInfo;
    }

    /**
     * 导出团队管理列表-不分页
     * @param teamId 团队id
     * @param keyword 模糊查询关键字
     * @return
     */
    @Override
    public List<VppTeamVO> exportManageTeamList(String teamId, String keyword) {
        List<String> manageAdminTeamIdList = null;
        if (StringUtils.isNotEmpty(keyword)) {
            keyword = keyword.concat("%");
            manageAdminTeamIdList = vppRTeamMemberRoleService.getTeamIdByAdminName(keyword);
        }
        VppMember member = vppMemberService.getCurrentUser();
        // 判断当前登录用户是否超管,过滤团队
        List<String> teamIdList = new ArrayList<>();
        if (!member.getRole().contains("1")) {
            // 团队管理员查找管理的所有团队
            List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(member.getId());
            teamIdList = vppRTeamMemberRoles.stream().filter(c -> c.getRoleId().equals("2")).map(VppRTeamMemberRole::getTeamId).collect(Collectors.toList());
        }
        List<VppTeamVO> vppTeamVOList = vppTeamMapper.getManageTeamList(member.getCompanyId(), teamId, keyword, teamIdList, manageAdminTeamIdList);
        // 查询团队成员数量和管理员信息
        for (VppTeamVO vppTeamVO : vppTeamVOList) {
            Integer memberAmount = 0;
            String teamAdmins = StringUtils.EMPTY;
            List<VppRTeamMemberRole> allByTeamId = vppRTeamMemberRoleService.getAllByTeamId(vppTeamVO.getId());
            if (!CollectionUtils.isEmpty(allByTeamId)) {
                memberAmount = allByTeamId.size();
                List<VppRTeamMemberRole> collect = allByTeamId.stream()
                        .filter(c -> (c.getRoleId().equals("1") || c.getRoleId().equals("2"))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(collect)) {
                    for (VppRTeamMemberRole vppRTeamMemberRole : collect) {
                        VppMember vppMember = vppMemberService.getById(vppRTeamMemberRole.getMemberId());
                        teamAdmins = teamAdmins.concat(vppMember.getEmail()).concat(";");
                    }
                }
            }
            vppTeamVO.setTeamAdmins(teamAdmins);
            vppTeamVO.setMemberAmount(memberAmount);
            vppTeamVO.setCreateTime(vppTeamVO.getCreateTime().substring(0, 19));

            // 统计团队序列码数量
            List<LicenseCodes> byTeamIds = vppLicenseCodeService.getByTeamIds(member.getCompanyId(), Collections.singletonList(vppTeamVO.getId()));
            List<LicenseCodes> collect = byTeamIds.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(byTeamIds) && CollectionUtils.isEmpty(collect)) {
                vppTeamVO.setLicenseAmount(0);
                vppTeamVO.setAvailableDevices(0);
                vppTeamVO.setActivatedDevices(0);
            } else {
                int total = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
                int used = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
                vppTeamVO.setLicenseAmount(byTeamIds.size());
                vppTeamVO.setAvailableDevices(total - used);
                vppTeamVO.setActivatedDevices(used);
            }
        }
        return vppTeamVOList;
    }

    @Override
    public void createTeam(CreateTeamDTO dto) {
        VppMember member = vppMemberService.getCurrentUser();
        // 获取公司默认团队
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andCompanyIdEqualTo(member.getCompanyId()).andTypeEqualTo(1);
        VppTeam vppTeam = vppTeamMapper.selectByExample(example).get(0);
        // 查询默认团队剩余未授权序列码,判断是否足够分配
        List<LicenseCodes> vppLicenseCodeList = null;
        int sum = 0;
        if (!CollectionUtils.isEmpty(dto.getProductList())) {
            vppLicenseCodeList = vppLicenseCodeService.getByCompanyAndTeamId(member.getCompanyId(), vppTeam.getId());
            for (CreateTeamDTO.Product product : dto.getProductList()) {
                List<LicenseCodes> collect = vppLicenseCodeList.stream().filter(c -> c.getProductId().equals(product.getProductId()) && (c.getValidFlag() == 1 || c.getValidFlag() == 4)).collect(Collectors.toList());
                if (collect.size() < product.getLicenseNumber()) {
                    throw new BackendRuntimeException(VppMemberConstant.INSUFFICIENT_QUANTITY_TO_ALLOCATE);
                }
            }
            sum = dto.getProductList().stream().mapToInt(CreateTeamDTO.Product::getLicenseNumber).sum();
        }

       String teamId = ObjectUtils.isEmpty(dto.getTeamId())?CommonUtils.generateId():dto.getTeamId();
        // 新增团队
        VppTeam newVppTeam = new VppTeam().withId(teamId).withValidFlag(ValidStatusEnum.VALID.value())
                .withCompanyId(member.getCompanyId()).withCreateTime(new Date());
        newVppTeam.setName(dto.getTeamName());
        newVppTeam.setDescription(null != dto.getTeamDescription() ? dto.getTeamDescription() : StringUtils.EMPTY);
        newVppTeam.setLicenseAmount(sum);
        newVppTeam.setLicenseRemain(newVppTeam.getLicenseAmount());
        newVppTeam.setType(ObjectUtils.isEmpty(dto.getType())?2:dto.getType());
        vppTeamMapper.insert(newVppTeam);

        if (!CollectionUtils.isEmpty(dto.getProductList())) {
            // 从默认团队分配序列码到此团队
            for (CreateTeamDTO.Product product : dto.getProductList()) {
                List<LicenseCodes> collect = vppLicenseCodeList.stream().filter(c -> c.getProductId().equals(product.getProductId()) && (c.getValidFlag() == 1 || c.getValidFlag() == 4)).collect(Collectors.toList());
                for (int i = 0; i < product.getLicenseNumber(); i++) {
                    LicenseCodes vppLicenseCode = collect.get(i);
                    vppLicenseCode.setTeamId(newVppTeam.getId());
                    vppLicenseCodeService.update(vppLicenseCode);
                }
            }
        }

        // 创建超管和团队关联关系
        VppMember superAdmin = vppMemberService.getSuperAdmin(member.getCompanyId());
        VppRTeamMemberRole vppRTeamMemberRoleSuper = new VppRTeamMemberRole().withId(CommonUtils.generateId())
                .withMemberId(superAdmin.getId()).withTeamId(newVppTeam.getId()).withRoleId("1");
        vppRTeamMemberRoleService.create(vppRTeamMemberRoleSuper);
        // 判断是否有管理员，没有直接返回
        if (CollectionUtils.isEmpty(dto.getTeamAdminId())) {
            return;
        }
        // 新增管理员
        for (String id : dto.getTeamAdminId()) {
            VppMember vppMember = vppMemberService.getById(id);
            CaptchaActionEnum actionEnum;
            String code = CommonUtils.generateVerifyCode();
            String text;
            String teamString = newVppTeam.getId();
            //如果已经注册账号需要判断该账号是普通用户还是管理员，如果是普通用户的话给一个密码
            VppCompany vppCompany = vppCompanyService.getById(superAdmin.getCompanyId());
            if(vppMember.getRole().equals("3") && StringUtils.isEmpty(vppMember.getDigestPassword())) {
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN_NO_PASSWORD;
                String pwd = CommonUtils.generateVerifyCode();
                vppMember.setDigestPassword(new RSAUtils().encrypt(pwd));
                vppMemberService.update(vppMember);
                //若管理员已注册账号，则仅发放邀请加入团队邮件；邮件带上memberId和用户的TeamId集合(emailTeamList)和code
                String url = webUrl + loginUrl + "?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 用户邮箱 密码 链接
                text = String.format(actionEnum.text(), vppMember.getFullName(), vppCompany.getName(),
                        vppMember.getEmail(), pwd, url);
            } else {
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN;
                String url = webUrl + loginUrl + "?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 链接
                text = String.format(actionEnum.text(), vppMember.getFullName(), vppCompany.getName(), url);
            }
            //发送邮件
            // 替换图片动态地址和取消订阅链接
            text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + vppMember.getEmail());
            String finalText = text;
            ThreadPoolSingleUtil.getInstance().excute(() -> {
                if(emailUtils.sendMail(actionEnum.subject(), finalText, Collections.singletonList(vppMember.getEmail()))){
                    redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + actionEnum.value(), vppMember.getEmail(), code, AuthConstant.INVITE_VERIFY_CODE_KEY_EXPIRE_TIME);
                }
            });
        }
    }

    @Override
    public void editTeam(EditTeamDTO dto) {
        // 获取编辑团队
        VppTeam vppTeam = vppTeamMapper.selectByPrimaryKey(dto.getTeamId());
        if (null == vppTeam) {
            throw new BackendRuntimeException(VppTeamConstant.TEAM_NOT_EXIST);
        }
        VppMember member = vppMemberService.getCurrentUser();
        // 判断当前登录用户是否超管,如果团管判断权限是否足够
        if (!member.getRole().contains("1")) {
            // 团队管理员查找管理的所有团队
            List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(member.getId());
            boolean flag = vppRTeamMemberRoles.stream().filter(c -> c.getRoleId().equals("2")).map(VppRTeamMemberRole::getTeamId).collect(Collectors.toList()).contains(vppTeam.getId());
            if (!flag) {
                throw new BackendRuntimeException(VppTeamConstant.INSUFFICIENT_PERMISSIONS);
            }
        }

        // 获取公司默认团队
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andCompanyIdEqualTo(member.getCompanyId()).andTypeEqualTo(1);
        VppTeam defaultVppTeam = vppTeamMapper.selectByExample(example).get(0);
        // 查询默认团队现有所有未分配序列码
        List<LicenseCodes> vppLicenseCodeList = vppLicenseCodeService.getByCompanyAndTeamId(member.getCompanyId(), defaultVppTeam.getId());
        // 查询团队所有序列码
        List<LicenseCodes> teamLicenseCodesList = vppLicenseCodeService.getByCompanyAndTeamId(vppTeam.getCompanyId(), vppTeam.getId());

        // 遍历参数,验证序列码（增加/删除）
        for (EditTeamDTO.Product product : dto.getProductList()) {
            // 获取产品序列码
            List<LicenseCodes> productLicenseList = teamLicenseCodesList.stream().filter(c -> c.getProductId().equals(product.getProductId())).collect(Collectors.toList());
            // 增加
            if (product.getLicenseNumber() > productLicenseList.size()) {
                // 查询默认团队剩余该产品序列码是否足够
                int count = (int) vppLicenseCodeList.stream().filter(c -> c.getProductId().equals(product.getProductId()) && (c.getValidFlag() == 1 || c.getValidFlag() == 4)).count();
                if (count < product.getLicenseNumber() - productLicenseList.size()) {
                    throw new BackendRuntimeException(VppMemberConstant.INSUFFICIENT_QUANTITY_TO_ALLOCATE);
                }
            } else if (product.getLicenseNumber() < productLicenseList.size()) {
                // 减少了序列码数量,判断未授权的团队序列码是否足够用来减
                List<LicenseCodes> collect = productLicenseList.stream().filter(c -> c.getValidFlag().equals(1) || c.getValidFlag().equals(4)).collect(Collectors.toList());
                int i = productLicenseList.size() - product.getLicenseNumber();
                if (collect.size() < i) {
                    throw new BackendRuntimeException(VppTeamConstant.ACTIVATED_LICENSE_CODE_EXCEEDED);
                }
            }
        }
        // 遍历团队序列码，验证参数中删除的
        List<Integer> deleteProductIdList = new ArrayList<>();
        List<Integer> productIdList = teamLicenseCodesList.stream().map(LicenseCodes::getProductId).distinct().collect(Collectors.toList());
        for (Integer productId : productIdList) {
            // 判断参数是否存在序列码id
            if (dto.getProductList().stream().noneMatch(c -> c.getProductId() != null && c.getProductId().equals(productId))) {
                // 参数不存在的产品序列码，判断该序列码是否已授权或激活
                if (teamLicenseCodesList.stream().filter(c -> c.getProductId().equals(productId)).anyMatch(c -> c.getValidFlag() == 2 || c.getValidFlag() == 3)) {
                    throw new BackendRuntimeException(VppTeamConstant.ACTIVATED_LICENSE_CODE_EXCEEDED);
                }
                deleteProductIdList.add(productId);
            }
        }

        // 编辑团队信息
        vppTeam.setName(dto.getTeamName());
        vppTeam.setDescription(dto.getTeamDescription());
        vppTeam.setLicenseAmount(dto.getProductList().stream().mapToInt(EditTeamDTO.Product::getLicenseNumber).sum());
        vppTeamMapper.updateByPrimaryKey(vppTeam);

        // 更新团队产品序列码
        for (EditTeamDTO.Product product : dto.getProductList()) {
            if (null == product.getProductId()) {
                continue;
            }
            // 获取本团队该产品序列码
            List<LicenseCodes> byTeamIdAndProduceId = vppLicenseCodeService.getByTeamIdAndProduceId(member.getCompanyId(), vppTeam.getId(), product.getProductId());
            if (byTeamIdAndProduceId.size() > product.getLicenseNumber()) {
                // 减少了序列码数量,判断未授权的团队序列码是否足够用来减
                List<LicenseCodes> collect = byTeamIdAndProduceId.stream().filter(c -> c.getValidFlag().equals(1) || c.getValidFlag().equals(4)).collect(Collectors.toList());
                int i = byTeamIdAndProduceId.size() - product.getLicenseNumber();
                if (collect.size() < i) {
                    throw new BackendRuntimeException(VppTeamConstant.ACTIVATED_LICENSE_CODE_EXCEEDED);
                }
                // 减少团队序列码
                for (int i1 = 0; i1 < i; i1++) {
                    LicenseCodes vppLicenseCode = collect.get(i1);
                    vppLicenseCode.setTeamId(defaultVppTeam.getId());
                    vppLicenseCodeService.update(vppLicenseCode);
                }
            } else if (byTeamIdAndProduceId.size() < product.getLicenseNumber()) {
                // 增加序列码数量，从默认团队分配多的序列码
                List<LicenseCodes> collect = vppLicenseCodeList.stream().filter(c -> c.getProductId().equals(product.getProductId()) && (c.getValidFlag() == 1 || c.getValidFlag() == 4)).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(collect)) {
                    throw new BackendRuntimeException(VppMemberConstant.INSUFFICIENT_QUANTITY_TO_ALLOCATE);
                }
                int a = product.getLicenseNumber() - byTeamIdAndProduceId.size();
                for (int i = 0; i < a; i++) {
                    LicenseCodes vppLicenseCode = collect.get(i);
                    vppLicenseCode.setTeamId(vppTeam.getId());
                    vppLicenseCodeService.update(vppLicenseCode);
                }
            }
        }
        // 从前端参数中删除的产品，数据处理到默认团队
        if (!CollectionUtils.isEmpty(deleteProductIdList)) {
            for (Integer productId : deleteProductIdList) {
                // 获取本团队该产品序列码
                List<LicenseCodes> teamLicenseList = vppLicenseCodeService.getByTeamIdAndProduceId(member.getCompanyId(), vppTeam.getId(), productId);
                for (LicenseCodes licenseCodes : teamLicenseList) {
                    licenseCodes.setTeamId(defaultVppTeam.getId());
                    vppLicenseCodeService.update(licenseCodes);
                }
            }
        }

        // 计算需要设置的团队管理员
        List<String> teamAdminId = new ArrayList<>();
        // 获取现有团队所有关联关系
        List<VppRTeamMemberRole> allByTeamId = vppRTeamMemberRoleService.getAllByTeamId(dto.getTeamId());
        if (CollectionUtils.isEmpty(allByTeamId)) {
            teamAdminId = dto.getTeamAdminId();
        } else {
            // 获取团队所有管理员
            List<VppRTeamMemberRole> vppRTeamMemberRoleList = allByTeamId.stream().filter(c -> c.getRoleId().equals("2")).collect(Collectors.toList());
            // 如果没设置过管理员
            if (CollectionUtils.isEmpty(vppRTeamMemberRoleList)) {
                teamAdminId = dto.getTeamAdminId();
            } else {
                // 已设置管理员用户id
                List<String> vppAdminIdList = vppRTeamMemberRoleList.stream().map(VppRTeamMemberRole::getMemberId).collect(Collectors.toList());
                // 判断管理员是否已设置
                for (String s : dto.getTeamAdminId()) {
                    // 前端参数不存在于现有团队管理员中，则需要设置管理员
                    if (!vppAdminIdList.contains(s)) {
                        teamAdminId.add(s);
                    }
                }
                // 判断删除了哪些管理员
                for (VppRTeamMemberRole vppRTeamMemberRole : vppRTeamMemberRoleList) {
                    // 现有团队管理员不存在于前端参数中，则删除管理员
                    if (!dto.getTeamAdminId().contains(vppRTeamMemberRole.getMemberId())) {
                        VppRTeamMemberRoleExample vppRTeamMemberRoleExample = new VppRTeamMemberRoleExample();
                        vppRTeamMemberRoleExample.createCriteria().andMemberIdEqualTo(vppRTeamMemberRole.getMemberId()).andTeamIdEqualTo(dto.getTeamId());
                        vppRTeamMemberRoleService.deleteByExample(vppRTeamMemberRoleExample);
                        // 获取管理源所有关联
                        List<VppRTeamMemberRole> memberRoles = vppRTeamMemberRoleService.listByMemberId(vppRTeamMemberRole.getMemberId());
                        // 1.判断管理员是否在其他团队有关联关系，如果没有则回到默认团队作为普通成员
                        if (CollectionUtils.isEmpty(memberRoles)) {
                            VppRTeamMemberRole vppRTeamMemberRole1 = new VppRTeamMemberRole().withId(CommonUtils.generateId())
                                    .withMemberId(vppRTeamMemberRole.getMemberId()).withTeamId(defaultVppTeam.getId()).withRoleId("3");
                            vppRTeamMemberRoleService.create(vppRTeamMemberRole1);
                            // 修改用户权限为普通成员
                            VppMember vppMember = new VppMember();
                            vppMember.setId(vppRTeamMemberRole.getMemberId());
                            vppMember.setRole("3");
                            vppMemberService.update(vppMember);
                        } else {
                            // 2.如果有关联关系。判断管理员是否还有其他管理的团队
                            List<VppRTeamMemberRole> collect = memberRoles.stream().filter(c -> c.getRoleId().equals("2")).collect(Collectors.toList());
                            // 如果没有管理的团队了，修改用户权限为普通成员
                            if (CollectionUtils.isEmpty(collect)) {
                                VppMember vppMember = new VppMember();
                                vppMember.setId(vppRTeamMemberRole.getMemberId());
                                vppMember.setRole("3");
                                vppMemberService.update(vppMember);
                            }
                        }
                        // 删除管理员去除该团队序列码关联
                        vppLicenseCodeService.unBindByMemberId(vppRTeamMemberRole.getMemberId(), Collections.singletonList(vppTeam.getId()));
                    }
                }
            }
        }

        // 设置管理员
//        String superAdmin = vppMemberService.getCompanySuperAdmin(member.getCompanyId());
        for (String id : teamAdminId) {
            VppMember vppMember = vppMemberService.getById(id);
            CaptchaActionEnum actionEnum;
            String code = CommonUtils.generateVerifyCode();
            String text;
            String teamString = vppTeam.getId();
            //如果已经注册账号需要判断该账号是普通用户还是管理员，如果是普通用户的话给一个密码
            VppCompany vppCompany = vppCompanyService.getById(member.getCompanyId());
            if(vppMember.getRole().equals("3") && StringUtils.isEmpty(vppMember.getDigestPassword())) {
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN_NO_PASSWORD;
                String pwd = CommonUtils.generateVerifyCode();
                vppMember.setDigestPassword(new RSAUtils().encrypt(pwd));
                vppMemberService.update(vppMember);
                //若管理员已注册账号，则仅发放邀请加入团队邮件；邮件带上memberId和用户的TeamId集合(emailTeamList)和code
                String url = webUrl + loginUrl + "?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 用户邮箱 密码 链接
                text = String.format(actionEnum.text(), vppMember.getFullName(), vppCompany.getName(),
                        vppMember.getEmail(), pwd, url);
            } else {
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN;
                String url = webUrl + loginUrl + "?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 链接
                text = String.format(actionEnum.text(), vppMember.getFullName(), vppCompany.getName(), url);
            }
            //发送邮件
            // 替换图片动态地址和取消订阅链接
            text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + vppMember.getEmail());
            String finalText = text;
            ThreadPoolSingleUtil.getInstance().excute(() -> {
                if(emailUtils.sendMail(actionEnum.subject(), finalText, Collections.singletonList(vppMember.getEmail()))){
                    redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + actionEnum.value(), vppMember.getEmail(), code, AuthConstant.INVITE_VERIFY_CODE_KEY_EXPIRE_TIME);
                }
            });
        }
    }

    @Override
    public TeamInfoVO getTeamProductInfo(String teamId) {
        // 获取团队
        VppTeam vppTeam = vppTeamMapper.selectByPrimaryKey(teamId);
        if (null == vppTeam) {
            throw new BackendRuntimeException(VppTeamConstant.TEAM_NOT_EXIST);
        }
        // 获取管理员
        List<VppTeamAdminVO> vppTeamAdminVOS = vppRTeamMemberRoleService.pageForAdmin(1, 100, teamId, null, null).getList();
        List<TeamInfoVO.Admin> teamAdminList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(vppTeamAdminVOS)) {
            for (VppTeamAdminVO vppTeamAdminVO : vppTeamAdminVOS) {
                TeamInfoVO.Admin admin = new TeamInfoVO.Admin();
                admin.setAdminId(vppTeamAdminVO.getId());
                admin.setAdminName(vppTeamAdminVO.getFullName());
                teamAdminList.add(admin);
            }
        }
        // 获取产品序列码
        List<LicenseCodes> byTeamIds = vppLicenseCodeService.getByCompanyAndTeamId(vppTeam.getCompanyId(), teamId);
        List<TeamInfoVO.Product> productList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(byTeamIds)) {
            // 根据产品id去重
            ArrayList<LicenseCodes> collect = byTeamIds.stream().collect(Collectors.collectingAndThen(
                    Collectors.toCollection(() -> new TreeSet<>(
                            Comparator.comparing(LicenseCodes::getProductId)
                    )), ArrayList::new
            ));
            for (LicenseCodes licenseCodes : collect) {
                TeamInfoVO.Product product = new TeamInfoVO.Product();
                product.setProductId(licenseCodes.getProductId());
                // 统计序列码数量
                int count = (int) byTeamIds.stream().filter(c -> c.getProductId().equals(licenseCodes.getProductId())).count();
                product.setLicenseNumber(count);
                productList.add(product);
            }
        }
        // 返回
        TeamInfoVO teamInfoVO = new TeamInfoVO();
        teamInfoVO.setTeamId(vppTeam.getId());
        teamInfoVO.setType(vppTeam.getType());
        teamInfoVO.setTeamName(vppTeam.getName());
        teamInfoVO.setTeamDescription(vppTeam.getDescription());
        teamInfoVO.setTeamAdminList(teamAdminList);
        teamInfoVO.setProductList(productList);
        return teamInfoVO;
    }

    @Override
    public void deleteTeam(String teamId) {
        // 获取团队信息
        VppTeam vppTeam = vppTeamMapper.selectByPrimaryKey(teamId);
        if (null == vppTeam) {
            throw new BackendRuntimeException(VppTeamConstant.TEAM_NOT_EXIST);
        }
        // 判断是否默认团队
        if (vppTeam.getType() == 1) {
            throw new BackendRuntimeException(VppTeamConstant.THE_DEFAULT_TEAM_CANNOT_BE_DELETED);
        }
        // 判断是否目录默认团队
        if (vppTeam.getType() == 3) {
            throw new BackendRuntimeException(VppTeamConstant.THE_DEFAULT_SSO_TEAM_CANNOT_BE_DELETED);
        }
        VppMember member = vppMemberService.getCurrentUser();
        // 判断当前登录用户是否超管,如果团管判断权限是否足够
        if (!member.getRole().contains("1")) {
            // 团队管理员查找管理的所有团队
            List<VppRTeamMemberRole> vppRTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(member.getId());
            boolean flag = vppRTeamMemberRoles.stream().filter(c -> c.getRoleId().equals("2")).map(VppRTeamMemberRole::getTeamId).collect(Collectors.toList()).contains(teamId);
            if (!flag) {
                throw new BackendRuntimeException(VppTeamConstant.INSUFFICIENT_PERMISSIONS);
            }
        }
//        // 判断团队是否还有已授权序列码数据
//        List<LicenseCodes> byTeamId = vppLicenseCodeService.getByTeamId(teamId);
//        if (!CollectionUtils.isEmpty(byTeamId)) {
//            throw new BackendRuntimeException(VppTeamConstant.EXIST_ACTIVATED_LICENSE_CODE);
//        }
        // 删除团队，返还未授权序列码到默认团队，移动团队成员到默认团队
        vppTeam.setValidFlag("0");
        vppTeamMapper.updateByPrimaryKeySelective(vppTeam);
        VppTeam defaultVppTeam = unbindDeviceANDCDKey(vppTeam, member);
        // 团队成员移到默认团队
        vppRTeamMemberRoleService.changeTeamMember(vppTeam.getId(), defaultVppTeam.getId());
        // 获取团队所有关联数据
        List<VppRTeamMemberRole> allByTeamId = vppRTeamMemberRoleService.getAllByTeamId(teamId);
        if (CollectionUtils.isEmpty(allByTeamId)) {
            return;
        }
        List<VppRTeamMemberRole> vppRTeamMemberRoles = allByTeamId.stream().filter(c -> c.getRoleId().equals("2")).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(vppRTeamMemberRoles)) {
            return;
        }
        // 删除团队所有关联关系
        vppRTeamMemberRoleService.cleanTeamMember(teamId);
        // 判断团队管理员逻辑
        for (VppRTeamMemberRole vppRTeamMemberRole : vppRTeamMemberRoles) {
            // 获取团队管理员
            List<VppRTeamMemberRole> memberRoles = vppRTeamMemberRoleService.listByMemberId(vppRTeamMemberRole.getMemberId());
            // 1.过滤每个管理员是否在其他团队有关联关系，如果没有则回到默认团队作为普通成员
            if (CollectionUtils.isEmpty(memberRoles)) {
                VppRTeamMemberRole vppRTeamMemberRole1 = new VppRTeamMemberRole().withId(CommonUtils.generateId())
                        .withMemberId(vppRTeamMemberRole.getMemberId()).withTeamId(defaultVppTeam.getId()).withRoleId("3");
                vppRTeamMemberRoleService.create(vppRTeamMemberRole1);
                // 修改用户权限为普通成员
                VppMember vppMember = new VppMember();
                vppMember.setId(vppRTeamMemberRole.getMemberId());
                vppMember.setRole("3");
                vppMemberService.update(vppMember);
            } else {
                // 2.如果有关联关系。判断管理员是否还有其他管理的团队
                List<VppRTeamMemberRole> collect = memberRoles.stream().filter(c -> c.getRoleId().equals("2")).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(collect)) {
                    // 如果没有管理的团队了，修改用户权限为普通成员
                    VppMember vppMember = new VppMember();
                    vppMember.setId(vppRTeamMemberRole.getMemberId());
                    vppMember.setRole("3");
                    vppMemberService.update(vppMember);
                }
            }
        }
    }

    /**
     * cdkey解绑 设备解绑
     * @param vppTeam 团队
     * @param member 用户
     * @return
     */
    private VppTeam unbindDeviceANDCDKey(VppTeam vppTeam, VppMember member) {
        // 获取公司默认团队
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andCompanyIdEqualTo(member.getCompanyId()).andTypeEqualTo(1);
        VppTeam defaultVppTeam = vppTeamMapper.selectByExample(example).get(0);
        // 获取团队所有序列码
        List<LicenseCodes> byCompanyAndTeamId = vppLicenseCodeService.getByCompanyAndTeamId(vppTeam.getCompanyId(), vppTeam.getId());
        List<String> cdKeys = new ArrayList<>();
        for (LicenseCodes vppLicenseCode : byCompanyAndTeamId) {
            // 已授权or已激活
            if (2 == vppLicenseCode.getValidFlag() || 3 == vppLicenseCode.getValidFlag()) {
                if (3 == vppLicenseCode.getValidFlag() && null != vppLicenseCode.getCdkey()) {
                    // 解绑激活设备
                    cdKeys.add(vppLicenseCode.getCdkey());
                }
                vppLicenseCode.setMemberId(null);
                vppLicenseCode.setVppMemberId(null);
                vppLicenseCode.setValidFlag(4);
                vppLicenseCode.setUpdatedAt(new Date());
                vppLicenseCode.setUsedTimes(Short.valueOf("0"));
            }
            vppLicenseCode.setTeamId(defaultVppTeam.getId());
            vppLicenseCodeService.updateByPrimaryKey(vppLicenseCode);
        }
        if (!CollectionUtils.isEmpty(cdKeys)) {
            vppDeviceService.unBindByMemberId(null, cdKeys);
        }
        return defaultVppTeam;
    }

    @Override
    public void sendEmailForAddTeamAdmin(List<String> teamIds, String account) {
        //判断用户存不存在，若被添加管理员暂未注册，则先发送邀请注册邮件，注册成功后直接加入团队
        VppMember vppMember = vppMemberService.getByAccount(account);
        if(!ObjectUtils.isEmpty(vppMember)&&RoleEnum.SUPER_ADMIN.code().equals(vppMember.getRole())){
            throw new RuntimeException(VppMemberConstant.EXCEPTION_MSG_USER_IS_SUPER_ADMIN);
        }
        CaptchaActionEnum actionEnum;
        String text;
        //发送邮件
        //判断用户存不存在，若被添加管理员暂未注册，则先发送邀请注册邮件，注册成功后直接加入团队
        VppMember member = vppMemberService.getCurrentUser();
        //用户存在其他公司时加入失败
        if(!ObjectUtils.isEmpty(vppMember)&&StringUtils.isNotEmpty(vppMember.getCompanyId())&&!vppMember.getCompanyId().equals(member.getCompanyId())){
            throw new RuntimeException(VppMemberConstant.EXCEPTION_MSG_USER_IS_IN_OTHER_COMPANY);
        }
//        String superAdmin = vppMemberService.getCompanySuperAdmin(member.getCompanyId());
        String teamString = "";
        for (String teamId : teamIds) {
            if(StringUtils.isEmpty(teamString)){
                teamString = teamId;
            }else{
                teamString = teamString + "," + teamId;
            }
        }
        String code = CommonUtils.generateVerifyCode();
        VppCompany vppCompany = vppCompanyService.getById(member.getCompanyId());
        if (ObjectUtils.isEmpty(vppMember)) {
            actionEnum = CaptchaActionEnum.USER_REGISTER_FOR_ADD_ADMIN;
            //发送注册邮件，前端根据注册页面表单信息和teamIds,还有code去进行注册
            String url = webUrl + createUrl +"?teamIds=" + teamString + "&code=" + code + "&action=" + actionEnum.value() + "&account=" + account;
            //用户邮箱 超管邮箱  链接
            text = String.format(CaptchaActionEnum.USER_REGISTER_FOR_ADD_ADMIN.text(), account, vppCompany.getName(), url, account);
        } else {
            // 判断是否有用户名，没有的用邮箱替换
            if (StringUtils.isEmpty(vppMember.getFullName())) {
                vppMember.setFullName(vppMember.getEmail());
            }
            // 用户管理的所有团队提取备用
            List<VppRTeamMemberRoleVO> rTeamMemberRoles = vppRTeamMemberRoleService.listByMemberIdAdmin(vppMember.getId());
            Map<String, List<VppRTeamMemberRoleVO>> teamMap = rTeamMemberRoles.stream().collect(Collectors.groupingBy(VppRTeamMemberRoleVO::getTeamId));
            List<String> emailTeamList = new ArrayList<>();
            teamIds.forEach(teamId -> {
                // 判断用户是否在某个团队,如果不在的话则记录团队id，用于发送邮件
                if (!teamMap.containsKey(teamId)) {
                    emailTeamList.add(teamId);
                }
            });
            //如果团队都已经管理了
//            if(CollectionUtils.isEmpty(emailTeamList)){
//                throw  new BackendRuntimeException(VppTeamConstant.USER_HAS_ADMIN_THAT_TEAM);
//            }
            //如果已经注册账号需要判断该账号是普通用户还是管理员，如果是普通用户的话给一个密码
            if(vppMember.getRole().contains("3")&&StringUtils.isEmpty(vppMember.getDigestPassword())){
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN_NO_PASSWORD;
                String pwd = CommonUtils.generateVerifyCode();
                vppMember.setDigestPassword(new RSAUtils().encrypt(pwd));
                vppMemberService.update(vppMember);
                //若管理员已注册账号，则仅发放邀请加入团队邮件；邮件带上memberId和用户的TeamId集合(emailTeamList)和code
                String url = webUrl + loginUrl +"?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 用户邮箱 密码 链接
                text = String.format(actionEnum.text(),vppMember.getFullName(),
                        vppCompany.getName(), vppMember.getEmail(), pwd, url);
            }else{
                actionEnum = CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN;
                String url = webUrl + loginUrl +"?teamIds=" + teamString + "&code=" + code + "&memberId=" + vppMember.getId() + "&action=" + actionEnum.value();
                //用户邮箱 超管邮箱 链接
                text = String.format(actionEnum.text(), vppMember.getFullName(), vppCompany.getName(), url);
            }
        }
        // 替换图片动态地址和取消订阅链接
        text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + account);
        String finalText = text;
        SEND_MAIL_THREAD_POOL.submit(() -> {
            //发送邮件
            if(emailUtils.sendMail(actionEnum.subject(), finalText, Collections.singletonList(account))){
                redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + actionEnum.value(), account, code, AuthConstant.INVITE_VERIFY_CODE_KEY_EXPIRE_TIME);
            }
        });
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editTeamAdmin(String memberId, List<String> teamIds) {
        VppMember vppMember = vppMemberService.getById(memberId);
        if (ObjectUtils.isEmpty(vppMember)) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_EMAIL_NOT_EXIST);
        }
        // 假如所有团队
        // 筛选出用户退出的团队
        // 用户加入的团队
        // 用户的所有团队提取备用
        List<VppRTeamMemberRole> rTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(vppMember.getId());
        Map<String, List<VppRTeamMemberRole>> teamMap = rTeamMemberRoles.stream().collect(Collectors.groupingBy(VppRTeamMemberRole::getTeamId));
        teamIds.forEach(teamId -> {
            //如果团队已经加入
            if (teamMap.containsKey(teamId)) {
                upRole(teamMap, teamId);
            } else {
                //不在团队的话就加入
                VppRTeamMemberRole vppRTeamMember = new VppRTeamMemberRole().withMemberId(memberId)
                        .withTeamId(teamId).withId(CommonUtils.generateId()).withRoleId(RoleEnum.TEAM_ADMIN.code());
                vppRTeamMemberRoleService.create(vppRTeamMember);
            }
        });
        List<String> quitTeamIds = new ArrayList<>();
        rTeamMemberRoles.forEach(roles ->  {
            //如果修改后的团队没有该团队则退出
            if (!teamIds.contains(roles.getTeamId())) {
                quitTeamIds.add(roles.getTeamId());
            }
        });
        if (CollectionUtils.isEmpty(teamIds)) {
            //如果没有任何团队了，则给他加入到默认团队，可以从默认团队把用户删除
            vppRTeamMemberRoleService.deleteAdmin(memberId);
            vppMemberService.updateMemberRole(memberId, RoleEnum.TEAM_MEMBER.code());
        } else {
            //退出团队
            if(!CollectionUtils.isEmpty(quitTeamIds)){
                vppRTeamMemberRoleService.quitTeam(memberId, quitTeamIds);
            }
            vppMemberService.updateMemberRole(memberId, RoleEnum.TEAM_ADMIN.code());
        }
    }

    private void upRole(Map<String, List<VppRTeamMemberRole>> teamMap, String teamId) {
        VppRTeamMemberRole roleVO = teamMap.get(teamId).get(0);
        // 如果用户在团队如果用户在该团队中则看用户的权限是不是管理员
        if (!RoleEnum.TEAM_ADMIN.code().equals(roleVO.getRoleId())) {
            //如果不是管理员就改成管理员 更新用户角色给与登录权限 如果是管理员则不用改动
            //更新用户和该团队的关联关系
            roleVO.setRoleId(RoleEnum.TEAM_ADMIN.code());
            vppRTeamMemberRoleService.update(roleVO);
        }
    }

    @Override
    public List<String> getMemberTeamIds() {
        List<VppTeam> vppTeams = listWithAdmin();
        return vppTeams.stream().map(VppTeam::getId).collect(Collectors.toList());
    }

    @Override
    public List<VppTeam> listWithAdmin() {
        VppMember vppMember = vppMemberService.getCurrentUser();
        if (vppMember.getRole().contains(RoleEnum.SUPER_ADMIN.code())) {
            //查询公司所有团队
            return listByCompany(vppMember.getCompanyId());
        }
        if (vppMember.getRole().contains(RoleEnum.TEAM_ADMIN.code())) {
            //查询自己管理的团队
            List<VppTeam> list = new ArrayList<>();
            List<VppRTeamMemberRoleVO> teamAdminslist = vppRTeamMemberRoleService.listByMemberIdAdmin(vppMember.getId());
            teamAdminslist.forEach(item -> {
                VppTeam vppTeam = new VppTeam().withId(item.getTeamId()).withName(item.getTeamName());
                list.add(vppTeam);
            });
            return list;
        }
        return new ArrayList<>();
    }


    @Override
    public void enterTeam(String memberId, List<String> teamIds, String action, String code) {
        VppMember member = vppMemberService.getById(memberId);
        CaptchaActionEnum captchaActionEnum = CaptchaActionEnum.getEnumByValue(action);
        //检查验证码
        authService.checkVerifyCodeByAction(captchaActionEnum, member.getEmail(), code);
        List<VppRTeamMemberRole> rTeamMemberRoles = vppRTeamMemberRoleService.listByMemberId(memberId);
        Map<String, List<VppRTeamMemberRole>> teamMap = rTeamMemberRoles.stream().collect(Collectors.groupingBy(VppRTeamMemberRole::getTeamId));
        teamIds.forEach(teamId -> {
            //如果团队已经加入
            if (teamMap.containsKey(teamId)) {
                upRole(teamMap, teamId);
            } else {
                //不在团队的话就加入
                VppRTeamMemberRole vppRTeamMember = new VppRTeamMemberRole().withMemberId(memberId)
                        .withTeamId(teamId).withId(CommonUtils.generateId());
                if (captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_MEMBER.value())) {
                    vppRTeamMember.withRoleId(RoleEnum.TEAM_MEMBER.code());
                }
                if (captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN.value())||captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN_NO_PASSWORD.value())) {
                    vppRTeamMember.withRoleId(RoleEnum.TEAM_ADMIN.code());
                }
                vppRTeamMemberRoleService.create(vppRTeamMember);
            }
        });
        VppTeam team = getById(teamIds.get(0));
        if(StringUtils.isEmpty(member.getCompanyId())){
            member.setCompanyId(team.getCompanyId());
        }
        member.setUpdatedAt(new Date());
        if (captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_MEMBER.value())) {
            updateMemberRole(member,RoleEnum.TEAM_MEMBER.code());
        }
        if (captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN.value())||captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN_NO_PASSWORD.value())) {
            updateMemberRole(member,RoleEnum.TEAM_ADMIN.code());
        }
        vppMemberService.update(member);
        //如果是添加管理员的邮件，用户之前只是普通用户，则给用户登录权限，并且给与一个密码
        if (captchaActionEnum.value().equals(CaptchaActionEnum.JOIN_TEAM_FOR_ADD_ADMIN.value())&&!member.getRole().contains(RoleEnum.TEAM_ADMIN.code())) {
            vppMemberService.updateMemberRole(memberId, RoleEnum.TEAM_ADMIN.code());
        }
    }

    public void updateMemberRole(VppMember vppMember, String role) {
        if (!vppMember.getRole().contains(role)) {
            if (ObjectUtils.isEmpty(vppMember.getRole())) {
                vppMember.setRole(role);
            } else {
                vppMember.setRole(vppMember.getRole() + "," + role);
            }
        }
    }

    @Override
    public List<VppTeam> listByCompany(String companyId) {
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andValidFlagEqualTo(ValidStatusEnum.VALID.value()).andCompanyIdEqualTo(companyId);
        return vppTeamMapper.selectByExample(example);
    }

    @Override
    public VppTeamListVO listForAdminEdit(String memberId) {
        VppTeamListVO vo = new VppTeamListVO();
        VppMember vppMember = vppMemberService.getCurrentUser();
        //获取用户管理的所有团队
        List<VppRTeamMemberRoleVO> teamAdminslist = vppRTeamMemberRoleService.listByMemberIdAdmin(memberId);
        List<String> manageTeamIds = teamAdminslist.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
        vo.setManageTeamIds(manageTeamIds);
        //超管管理的所有团队
        vo.setVppTeams(listByCompany(vppMember.getCompanyId()));
        return vo;
    }


    @Override
    public void updateTeam(VppTeam vppTeam) {
        vppTeamMapper.updateByPrimaryKeySelective(vppTeam);
    }

    @Override
    public VppTeam getById(String teamId) {
        return vppTeamMapper.selectByPrimaryKey(teamId);
    }

    @Override
    public List<VppTeam> getByTeamParam(String companyId, String teamId, List<String> licenseTeamIdList, List<String> manageTeamIdList) {
        return vppTeamMapper.getByTeamParam(companyId, teamId, licenseTeamIdList, manageTeamIdList);
    }

    @Override
    public VppTeam getByCompanyAndName(String companyId, String teamName) {
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andCompanyIdEqualTo(companyId).andNameEqualTo(teamName)
                .andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        List<VppTeam> vppTeams = vppTeamMapper.selectByExample(example);
        return CollectionUtils.isEmpty(vppTeams) ? null : vppTeams.get(0);
    }

    @Override
    public VppTeam getDefaultTeam(String memberId) {
        VppMember vppMember = vppMemberService.getById(memberId);
        VppTeamExample example = new VppTeamExample();
        example.createCriteria().andCompanyIdEqualTo(vppMember.getCompanyId()).andValidFlagEqualTo(ValidStatusEnum.VALID.value()).andTypeEqualTo(1);
        return vppTeamMapper.selectByExample(example).get(0);
    }

    @Override
    public void deleteDirectoryTeam(String teamId) {
        // 删除团队
        VppTeam vppTeam = vppTeamMapper.selectByPrimaryKey(teamId);
        vppTeam.setValidFlag(ValidStatusEnum.INVALID.value());
        vppTeamMapper.updateByPrimaryKeySelective(vppTeam);
        // 解绑设备
        // 归还序列码
        VppMember member = vppMemberService.getCurrentUser();
        unbindDeviceANDCDKey(vppTeam, member);
        // 删除团队所有关联关系
        vppRTeamMemberRoleService.cleanTeamMember(teamId);

    }

}
