package cn.kdan.pdf.tech.core.task;

import cn.kdan.pdf.tech.core.enums.DeviceStatusEnum;
import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.model.OemLicenseCode;
import cn.kdan.pdf.tech.core.service.OemDeviceService;
import cn.kdan.pdf.tech.core.service.OemLicenseCodeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
@RequiredArgsConstructor
public class LicenseTask {
    private final OemLicenseCodeService licenseCodeService;
    private final OemDeviceService vppDeviceService;

    @Scheduled(fixedRate = 3 * 60 * 1000) // 每3小时执行一次
    public void handleLicenseCode() {
        List<OemLicenseCode> licenseCodes = licenseCodeService.listActive();
        //已处理过过期的订阅id
        List<String> list = new ArrayList<>();
        //验证序列码有效性
        //如果过期时间小于当前时间，则置为过期，并且修改订阅状态，序列码状态，设备状态
        licenseCodes.stream().filter(licenseCode -> licenseCode.getEndUpAt() != null).forEach(licenseCode -> {
            if (licenseCode.getEndUpAt().getTime() < System.currentTimeMillis()) {
                //过期序列码置为过期
                OemLicenseCode expiredLicenseCode = new OemLicenseCode();
                expiredLicenseCode.setId(licenseCode.getId());
                expiredLicenseCode.setValidFlag(LicenseCodeStatusEnum.EXPIRED.code());
                licenseCodeService.update(expiredLicenseCode);
                log.error("序列码证时间过期，序列码id:{}，过期", licenseCode.getId());
                vppDeviceService.expireDevice(licenseCode.getCdkey(),expiredLicenseCode.getType().equals(Short.valueOf("1"))?DeviceStatusEnum.EXPIRED:DeviceStatusEnum.EXPIRED_ON_TRIAL);
            }
        });
    }

}
