package cn.kdan.pdf.tech.core.utils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 计算持续时间
 * @author tangxiangan
 */
public class PeriodUtils {


    /**
     * 将date与ruby中的period类型（1.days、1.months、1.years）数据进行相加
     *
     * @param date   操作时间
     * @param period 增加时间数
     * @return 返回被增加后的时间
     */
    public static Date countDate(Date date, String period) {
        LocalDateTime localDateTime = covertLocalDateTime(date);
        List<String> periods = Arrays.stream(period.split("\\.")).collect(Collectors.toList());
        String num = periods.get(0);
        String unit = periods.get(1);
        if ("month".equals(unit) || "months".equals(unit)) {
            localDateTime = localDateTime.plusMonths(Long.parseLong(num));
        } else if ("day".equals(unit) || "days".equals(unit)) {
            localDateTime = localDateTime.plusDays(Long.parseLong(num));
        } else if ("year".equals(unit) || "years".equals(unit)) {
            localDateTime = localDateTime.plusYears(Long.parseLong(num));
        }
        return covertDate(localDateTime);
    }

    /**
     * Date时间格式转LocalDateTime
     *
     * @param date 需要转的日期Date
     * @return 返回被转后的LocalDateTime
     */
    public static LocalDateTime covertLocalDateTime(Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        return instant.atZone(zoneId).toLocalDateTime();
    }

    /**
     * 将LocalDateTime数据转换成Date
     *
     * @param time 需要转的LocalDateTime
     * @return 转换后的Date
     */
    public static Date covertDate(LocalDateTime time) {
        ZoneId zoneId = ZoneId.systemDefault();
        ZonedDateTime zdt = time.atZone(zoneId);
        return Date.from(zdt.toInstant());
    }

}
